// Include-Dateien
#include "DirectDraw.h"
#include "Globals.h"

// Globale Variablen
LPDIRECTDRAW7			g_pDirectDraw = NULL;
LPDIRECTDRAWSURFACE7	g_pPrimarySurface = NULL;
LPDIRECTDRAWSURFACE7	g_pBackBuffer = NULL;
LPDIRECTDRAWCLIPPER		g_pClipper = NULL;
DDSURFACEDESC2			g_DDSD;

// Struktur fr ein Clipping-Rechteck (DirectDraw)
struct SClipList
{
	RGNDATAHEADER Header;
	RECT aRect[1];
};

// Initialisiert DirectDraw
bool InitDirectDraw(int iResX,
					int iResY,
					int iBPP,
					HWND hWindow)
{
	DDSURFACEDESC2 DDSD;
	SClipList ClipList;


	// Schnittstelle generieren, Kooperationsebene und Videomodus setzen
	if(FAILED(DirectDrawCreateEx(NULL, (void**)(&g_pDirectDraw), IID_IDirectDraw7, NULL))) return false;
	if(FAILED(g_pDirectDraw->SetCooperativeLevel(hWindow, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN))) return false;
	if(FAILED(g_pDirectDraw->SetDisplayMode(iResX, iResY, iBPP, 0, 0))) return false;

	// Primre Oberflche erstellen
	ZeroMemory(&DDSD, sizeof(DDSURFACEDESC2));
	DDSD.dwSize = sizeof(DDSURFACEDESC2);
	DDSD.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
	DDSD.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
	DDSD.dwBackBufferCount = 1;
	if(FAILED(g_pDirectDraw->CreateSurface(&DDSD, &g_pPrimarySurface, NULL))) return false;

	// Back-Buffer erstellen
	DDSD.ddsCaps.dwCaps = DDSCAPS_BACKBUFFER;
	if(FAILED(g_pPrimarySurface->GetAttachedSurface(&DDSD.ddsCaps, &g_pBackBuffer))) return false;

	// Beschreibung des Back-Buffers abfragen
	ZeroMemory(&g_DDSD, sizeof(DDSURFACEDESC2));
	g_DDSD.dwSize = sizeof(DDSURFACEDESC2);
	if(FAILED(g_pBackBuffer->GetSurfaceDesc(&g_DDSD))) return false;

	// Clipper erstellen
	if(FAILED(g_pDirectDraw->CreateClipper(0, &g_pClipper, NULL))) return false;

	// Dem Clipper das Clipping-Rechteck zuweisen
	ZeroMemory(&ClipList, sizeof(SClipList));
	ClipList.Header.dwSize = sizeof(RGNDATAHEADER);
	ClipList.Header.iType = RDH_RECTANGLES;
	ClipList.Header.nCount = 1;
	ClipList.Header.nRgnSize = 0;
	SetRect(&ClipList.Header.rcBound, 0, 0, iResX, iResY);
	SetRect(&ClipList.aRect[0], 0, 0, iResX, iResY);
	g_pClipper->SetClipList((RGNDATA*)(&ClipList), 0);

	// Dem Back-Buffer den Clipper zuweisen
	g_pBackBuffer->SetClipper(g_pClipper);

	return true;
}

// Herunterfahren von DirectDraw
bool ExitDirectDraw()
{
	if(g_pDirectDraw != NULL) g_pDirectDraw->RestoreDisplayMode();
	if(g_pBackBuffer != NULL) g_pBackBuffer->SetClipper(NULL);
	SAFE_RELEASE(g_pClipper);
	SAFE_RELEASE(g_pBackBuffer);
	SAFE_RELEASE(g_pPrimarySurface);
	SAFE_RELEASE(g_pDirectDraw);

	return true;
}

// Zeigt den Back-Buffer an
bool ShowBackBuffer(bool bVSync)
{
	// Auf vertikalen Strahlrcklauf warten (wenn erwnscht), und den Back-Buffer sichtbar machen
	if(bVSync) g_pDirectDraw->WaitForVerticalBlank(DDWAITVB_BLOCKBEGIN, NULL);
	if(FAILED(g_pPrimarySurface->Blt(NULL, g_pBackBuffer, NULL, DDBLT_WAIT, NULL))) return true;

	return true;
}

// Leert eine Oberflche
bool ClearSurface(LPDIRECTDRAWSURFACE7 pSurface,
				  DWORD dwColor)
{
	DDBLTFX FX;

	ZeroMemory(&FX, sizeof(DDBLTFX));
	FX.dwSize = sizeof(DDBLTFX);
	FX.dwFillColor = dwColor;

	if(FAILED(pSurface->Blt(NULL, NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &FX))) return false;

	return true;
}

// Ldt ein Bitmap und liefert die Oberflche
bool LoadBitmap(char* pcFilename,
				LPDIRECTDRAWSURFACE7* ppSurface,
				DWORD dwColorKey)
{
	HDC hBackBufferDC;
	HDC hBmpDC;
	HDC hSurfaceDC;
	HBITMAP hBmp;
	BITMAP Bitmap;
	DDSURFACEDESC2 DDSD;
	DDCOLORKEY ColorKey;


	// DC fr das Bild erstellen, kompatibel zum Back-Buffer
	if(FAILED(g_pBackBuffer->GetDC(&hBackBufferDC))) return false;
	hBmpDC = CreateCompatibleDC(hBackBufferDC);
	g_pBackBuffer->ReleaseDC(hBackBufferDC);
	if(hBmpDC == NULL) return false;

	// Bild laden und in die DC setzen
	hBmp = (HBITMAP)(LoadImage(GetModuleHandle(NULL), pcFilename, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE));
	if(hBmp == NULL) {DeleteDC(hBmpDC); return false;}
	GetObject(hBmp, sizeof(BITMAP), &Bitmap);
	SelectObject(hBmpDC, hBmp);

	// Oberflche erstellen, wenn mglich im Videospeicher
	ZeroMemory(&DDSD, sizeof(DDSURFACEDESC2));
	DDSD.dwSize = sizeof(DDSURFACEDESC2);
	DDSD.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
	DDSD.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
	DDSD.dwWidth = Bitmap.bmWidth;
	DDSD.dwHeight = Bitmap.bmHeight;
	if(FAILED(g_pDirectDraw->CreateSurface(&DDSD, ppSurface, NULL)))
	{
		DDSD.ddsCaps.dwCaps ^= DDSCAPS_VIDEOMEMORY;
		if(FAILED(g_pDirectDraw->CreateSurface(&DDSD, ppSurface, NULL)))
		{
			DeleteDC(hBmpDC);
			DeleteObject(hBmp);
			return false;
		}
	}

	// DC der Oberflche abfragen
	if(FAILED((*ppSurface)->GetDC(&hSurfaceDC)))
	{
		(*ppSurface)->Release();
		DeleteDC(hBmpDC);
		DeleteObject(hBmp);
		return false;
	}

	// Bild hineinkopieren
	BitBlt(hSurfaceDC, 0, 0, Bitmap.bmWidth, Bitmap.bmHeight, hBmpDC, 0, 0, SRCCOPY);
	(*ppSurface)->ReleaseDC(hSurfaceDC);

	// Color-Key setzen
	ColorKey.dwColorSpaceLowValue = dwColorKey;
	ColorKey.dwColorSpaceHighValue = dwColorKey;
	(*ppSurface)->SetColorKey(DDCKEY_SRCBLT, &ColorKey);

	SelectObject(hBmpDC, NULL);
	DeleteDC(hBmpDC);
	DeleteObject(hBmp);

	return true;
}

// Zeichnet ein Sprite
bool DrawSprite(LPDIRECTDRAWSURFACE7 pDest,		  
				int dx,
				int dy,
				int dw,
				int dh,
				LPDIRECTDRAWSURFACE7 pSpriteSet,
				int sx,
				int sy,
				int sw,
				int sh,
				bool bTransparent)
{
	RECT DestRect;
	RECT SrcRect;


	if(dw == -1) dw = sw;
	if(dh == -1) dh = sh;
	SetRect(&DestRect, dx, dy, dx + dw, dy + dh);
	SetRect(&SrcRect, sx, sy, sx + sw, sy + sh);

	if(FAILED(pDest->Blt(&DestRect, pSpriteSet, &SrcRect, bTransparent ? DDBLT_KEYSRC : 0, NULL))) return false;
	return true;
}

// Erweiterte Blt-Funktion
bool Blt(LPDIRECTDRAWSURFACE7 pDest,
		 int dx,
		 int dy,
		 int dw,
		 int dh,
		 LPDIRECTDRAWSURFACE7 pSrc,
		 int sx,
		 int sy,
		 int sw,
		 int sh,
		 DWORD dwFlags,
		 DDBLTFX* pFX)
{
	DDSURFACEDESC2 DestDDSD;
	DDSURFACEDESC2 SrcDDSD;
	RECT DestRect;
	RECT SrcRect;


	ZeroMemory(&DestDDSD, sizeof(DDSURFACEDESC2)); DestDDSD.dwSize = sizeof(DDSURFACEDESC2);
	ZeroMemory(&SrcDDSD, sizeof(DDSURFACEDESC2)); SrcDDSD.dwSize = sizeof(DDSURFACEDESC2);
	pDest->GetSurfaceDesc(&DestDDSD);
	pSrc->GetSurfaceDesc(&SrcDDSD);

	if(dw == -1) dw = DestDDSD.dwWidth;
	if(dh == -1) dh = DestDDSD.dwHeight;
	if(dw == -2) dw = SrcDDSD.dwWidth;
	if(dh == -2) dh = SrcDDSD.dwHeight;
	if(dw == -3) dw = DestDDSD.dwWidth - dx;
	if(dh == -3) dh = DestDDSD.dwHeight - dy;
	if(sw == -1) sw = SrcDDSD.dwWidth;
	if(sh == -1) sh = SrcDDSD.dwHeight;
	if(sw == -2) sw = DestDDSD.dwWidth;
	if(sh == -2) sh = DestDDSD.dwHeight;
	if(sw == -3) sw = SrcDDSD.dwWidth - sx;
	if(sh == -3) sh = SrcDDSD.dwHeight - sy;

	SetRect(&DestRect, dx, dy, dx + dw, dy + dh);
	SetRect(&SrcRect, sx, sy, sx + sw, sy + sh);

	if(FAILED(pDest->Blt(&DestRect, pSrc, &SrcRect, dwFlags, pFX))) return false;
	return true;
}