// Include-Dateien
#include "DirectSound.h"
#include "Globals.h"
#include <StdIO.h>

// Globale Variablen
LPDIRECTSOUND		g_pDirectSound;
LPDIRECTSOUNDBUFFER	g_pPrimaryBuffer;

// DirectSound initialisieren
bool InitDirectSound(HWND hWindow)
{
	DSBUFFERDESC BufferDesc;

	// Schnittstelle erzeugen und Kooperationsebene setzen
	if(FAILED(DirectSoundCreate(NULL, &g_pDirectSound, NULL))) return false;
	g_pDirectSound->SetCooperativeLevel(hWindow, DSSCL_NORMAL);

	// Primren Soundpuffer erstellen
	ZeroMemory(&BufferDesc, sizeof(DSBUFFERDESC));
	BufferDesc.dwSize = sizeof(DSBUFFERDESC);
	BufferDesc.dwFlags = DSBCAPS_PRIMARYBUFFER;
	if(FAILED(g_pDirectSound->CreateSoundBuffer(&BufferDesc, &g_pPrimaryBuffer, NULL))) return false;

	return true;
}

// DirectSound herunterfahren
bool ExitDirectSound()
{
	SAFE_RELEASE(g_pPrimaryBuffer);
	SAFE_RELEASE(g_pDirectSound);

	return true;
}

// Ldt einen Sound und liefert den Soundpuffer
bool LoadSound(char* pcFilename,
			   DWORD dwFlags,
			   LPDIRECTSOUNDBUFFER* ppOut)
{
	FILE*			pFile;
	SRIFFHeader		RIFFHeader;
	SWAVChunkHeader	ChunkHeader;
	BOOL			bFormatChunkRead = FALSE;
	BOOL			bDataChunkRead = FALSE;
	DWORD			dwNumBytesToRead;
	WAVEFORMATEX	WaveFormat;
	void*			pSoundData = NULL;
	DSBUFFERDESC	BufferDesc;
	void*			pSoundBufferData;
	DWORD			dwSoundBufferDataSize;
	DWORD			dwSoundBufferSize;


	// WAV-Datei ffnen
	pFile = fopen(pcFilename, "rb");
	if(pFile == NULL) return false;

	// Header lesen und sicherstellen, dass es wirklich eine WAV-Datei ist
	fread(&RIFFHeader, sizeof(SRIFFHeader), 1, pFile);
	if(strnicmp(RIFFHeader.acRIFF, "RIFF", 4)) return false;
	if(strnicmp(RIFFHeader.acFormat, "WAVE", 4)) return false;

	while(true)
	{
		// Chunk-Header lesen
		if(fread(&ChunkHeader, sizeof(SWAVChunkHeader), 1, pFile) != 1) break;

		// Format-Chunk?
		if(!strnicmp(ChunkHeader.acType, "fmt ", 4))
		{
			// Format lesen
			dwNumBytesToRead = min(ChunkHeader.dwDataSize, sizeof(WAVEFORMATEX));
			if(fread(&WaveFormat, dwNumBytesToRead, 1, pFile) != 1) break;
			bFormatChunkRead = TRUE;
		}
		// Daten-Chunk?
		else if(!strnicmp(ChunkHeader.acType, "data", 4))
		{
			// Daten lesen
			pSoundData = malloc(ChunkHeader.dwDataSize);
			dwSoundBufferSize = ChunkHeader.dwDataSize;
			if(pSoundData == NULL) return false;
			if(fread(pSoundData, ChunkHeader.dwDataSize, 1, pFile) != 1) break;
			bDataChunkRead = TRUE;
		}
		else
		{
			// Chunk berspringen
			fseek(pFile, ChunkHeader.dwDataSize, SEEK_CUR);
		}
	}

	// Wir brauchen einen Format- und einen Daten-Chunk
	if(!(bFormatChunkRead && bDataChunkRead))
	{
		free(pSoundData);
		return false;
	}

	// Pufferbeschreibung ausfllen
	BufferDesc.dwSize			= sizeof(DSBUFFERDESC);
	BufferDesc.dwFlags			= dwFlags;
	BufferDesc.dwBufferBytes	= dwSoundBufferSize;
	BufferDesc.dwReserved		= 0;
	BufferDesc.lpwfxFormat		= &WaveFormat;
	BufferDesc.guid3DAlgorithm	= GUID_NULL;

	// Soundpuffer erstellen
	if(FAILED(g_pDirectSound->CreateSoundBuffer(&BufferDesc,
												ppOut,
												NULL)))
	{
		free(pSoundData);
		return false;
	}

	// Puffer sperren und Daten hineinkopieren
	(*ppOut)->Lock(0, 0, &pSoundBufferData, &dwSoundBufferDataSize, NULL, NULL, DSBLOCK_ENTIREBUFFER);
	memcpy(pSoundBufferData, pSoundData, min(dwSoundBufferSize, (int)(dwSoundBufferDataSize)));
	(*ppOut)->Unlock(pSoundBufferData, dwSoundBufferDataSize, NULL, 0);

	free(pSoundData);

	return true;
}