// Include-Dateien
#include "Window.h"
#include "DirectDraw.h"
#include "DirectSound.h"
#include "Globals.h"
#include "Resource.h"
#include <MMSystem.h>
#include <WinINet.h>
#include <IO.h>
#include <StdIO.h>
#include <Math.h>

// Struktur fr ein Tile
struct STile
{
	unsigned char x;
	unsigned char y;
};

// Struktur fr den Spieler
struct SPlayer
{
	float	x;				// x-Position
	float	y;				// y-Position
	float	vx;				// x-Bewegung
	float	vy;				// y-Bewegung
	float	fAnim;			// Animationsphase
	float	fFallAnim;		// Animationsphase fr's Fallen
	float	fJumpCD;		// Countdown fr Sprnge
	int		iDir;			// Blickrichtung (-1: links; +1: rechts)
	BOOL	bStanding;		// Steht der Spieler?
	BOOL	bJumping;		// Springt der Spieler?
	BOOL	bReadyToJump;	// Bereit zum Springen?
	float	fMoney;			// Geld als float-Wert
	int		iMoney;			// Geld als int-Wert
};

// Struktur fr einen Gegenstandstyp
struct SItemType
{
	int		sy;				// Zeile im Bild
	int		iValue;			// Wert
	int		iMagnet;		// Magnet?
	BOOL	bRoulette;		// Zufallswert?
	int		iFlyingMode;	// Flugverhalten
	int		iNumAnimFrames;	// Anzahl der Frames in der Animation
	float	fAnimSpeed;		// Animationsgeschwindigkeit
	int		iEx1;			// Erweiterter Wert 1
	int		iEx2;			// Erweiterter Wert 2
};

// Struktur fr einen Gegenstand
struct SItem
{
	BOOL		bExists;	// Existiert er?
	int			iType;		// Typ (Nummer)
	SItemType*	pType;		// Typ (Zeiger auf SItemType)
	int			iValue;		// Wert
	BOOL		bExploding;	// Explodiert er gerade?
	BOOL		bCaught;	// Wurde er gefangen (wird er nur noch als Geld angezeigt)?
	float		x;
	float		y;
	float		vx;			
	float		vy;
	float		fAnim;		// Animationsphase
};

struct SHSEntry
{
	DWORD		dwSum;
	char		acName[256];
	SYSTEMTIME	TimeDate;
	int			iMoney;
	int			iSecurity1;
	float		fSecurity2;
	float		fSecurity3;
};

// Optionen
struct SOptions
{
	BOOL bShowSky;
	BOOL bShowSnow;
	BOOL bUseSound;
	BOOL bUseMusic;
	BOOL bNoHiPrio;
};

// Schneeflocke
struct SSnowflake
{
	float	x;
	float	y;
	float	z;
	int		iType;
};

// Aufzug
struct SElevator
{
	int		iSpriteX;		// x-Position des Sprites in Misc.bmp
	int		iSpriteY;		// y-Position des Sprites in Misc.bmp
	int		iWidth;			// Breite
	int		iHeight;		// Hhe
	float	x;
	float	y;
	float	vx;
	float	vy;
	float	afPointX[16];	// Max. 16 Punkte, die er abfhrt
	float	afPointY[16];	// "
	float	fSpeed;			// Geschwindigkeit
	float	fTimeLeft;		// Zeit bis zum nchsten Haltepunkt
	int		iNumPoints;		// Anzahl der Haltepunkte
	int		iCurrentDest;	// Aktueller Zielpunkt
};

// Globale Variablen
LPDIRECTDRAWSURFACE7	g_pTiles = NULL;					// Oberflche fr die Tiles
LPDIRECTDRAWSURFACE7	g_pSky1 = NULL;						// 1. Himmel
LPDIRECTDRAWSURFACE7	g_pSky2 = NULL;						// 2. Himmel
LPDIRECTDRAWSURFACE7	g_pTitle = NULL;					// Titelbild
LPDIRECTDRAWSURFACE7	g_pRicardo = NULL;					// Die Spielfigur
LPDIRECTDRAWSURFACE7	g_pItems = NULL;					// Die Gegenstnde
LPDIRECTDRAWSURFACE7	g_pMisc = NULL;						// Andere Dinge (z.B. Schneeflocken, Aufzge...)
HFONT					g_hFont1 = NULL;					// 1. Schriftart
HFONT					g_hFont2 = NULL;					// 2. Schriftart
HCURSOR					g_hCursor = NULL;					// Cursor
HCURSOR					g_hWaitCursor = NULL;				// Cursor beim Warten (Online-High-Scores)
char					g_acLevel[256];						// Aktueller Level (Dateiname)
int						g_iLevel = 0;						// Aktueller Level (Nummer)
int						g_iLevelWidth;						// Breite des Levels
int						g_iLevelHeight;						// Hhe des Levels
int						g_iSkyZ1;							// Tiefe des ersten Himmels
int						g_iSkyZ2;							// Tiefe des zweiten Himmels
SPlayer					g_Player;							// Spieler
float					g_fCameraX;							// x-Kameraposition
float					g_fCameraY;							// y-Kameraposition
int						g_aiLevel[256][256];				// Level als 2D-Array
int						g_aiTileType[1024];					// Tile-Typen
SItemType				g_aItemType[256];					// Gegenstandstypen
int						g_aiFreqTable[100];					// Wahrscheinlichkeitstabelle fr Gegenstnde
SItem					g_aItem[1024];						// Gegenstnde
int						g_iTotalItems;						// Anzahl der Gegenstnde
float					g_fTime;							// Zeitzhler
float					g_fCD;								// Countdown fr Level
float					g_fGravity;							// Gravitation
float					g_fMinItemSpeed;					// Min. Geschwindigkeit der Gegenstnde
SOptions				g_Options;							// Optionen
DWORD					g_dwLastMouseMove;					// Wann die Maus zum letzten Mal bewegt wurde
int						g_iSnow;							// Scheit es?
int						g_iSnowSpriteX;						// x-Position der Schnee-Sprites
int						g_iSnowSpriteY;						// y-Position der Schnee-Sprites
int						g_iSnowSpriteSize;					// Gre der Schnee-Sprites
int						g_iNumSnowSprites;					// Anzahl der Schnee-Sprites
int						g_iSnowDensity;						// Schneedichte
SSnowflake				g_aSnowflake[4096];					// Schneeflocken
int						g_iNumElevators;					// Anzahl der Aufzge
SElevator				g_aElevator[256];					// Aufzge
float					g_fPlayerSpeed;						// Spielergeschwindigkeit
float					g_fFrictionX;						// Reibung in x-Richtung
float					g_fFrictionY;						// Reibung in y-Richtung
int						g_iNumTilesPerRow;					// Tiles pro Zeile
float					g_fPlayerJumpFactor;				// Sprungfaktor (hher/weiter)
const int				g_iNumSounds = 23;					// Anzahl der Sounds
LPDIRECTSOUNDBUFFER		g_apSound[g_iNumSounds] = {NULL};	// Sounds
char					g_acMusic[256];						// Musikdateiname
BOOL					g_bEnglish = FALSE;					// Spiel in Englisch?

// Funktionen vordeklarieren
bool MainMenu();
bool SelectLevel();
bool Play();

// Inline-Funktionen zum Abfragen der x- und y-Position eines Tiles
inline int GetTileX(int iTile) {return (iTile % g_iNumTilesPerRow) * 32;}
inline int GetTileY(int iTile) {return (iTile / g_iNumTilesPerRow) * 32;}

DWORD GetFileCS(char* pcFilename)
{
	FILE*	pFile;
	BYTE	Byte;
	DWORD	dwSum = 0;
	DWORD	dwSize;
	HANDLE	hFile;
	int		iByte = 0;


	pFile = fopen(pcFilename, "rb");
	if(pFile == NULL) return 0;
	
	while(!feof(pFile))
	{
		fread(&Byte, sizeof(BYTE), 1, pFile);
		dwSum += Byte;
		dwSum += Byte * iByte;
		iByte++;
	}
	
	fclose(pFile);

	hFile = CreateFile(pcFilename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_ALWAYS,
	                   FILE_ATTRIBUTE_READONLY, NULL);
	GetFileSize(hFile, &dwSize);
	dwSum += dwSize;
	CloseHandle(hFile);

	return dwSum;
}

DWORD GetLevelCS(char* pcFilename)
{
	FILE*	pFile;
	char	acKey[256];
	char	acValue[256];
	char	acText[256];
	DWORD	dwSum = 0;


	dwSum = GetFileCS(pcFilename);

	sprintf(acText, "Data\\%s", pcFilename);
	pFile = fopen(acText, "rb");
	if(pFile == NULL) return 0;

	while(!feof(pFile))
	{
		fscanf(pFile, "%s = %s", acKey, acValue);

		if(!strcmp(acKey, "MapFile") ||
		   !strcmp(acKey, "TilesData") ||
		   !strcmp(acKey, "ItemsData"))
		{
			strcpy(acText, acValue);
			sprintf(acValue, "Data\\%s", acText);
			dwSum += GetFileCS(acValue);
		}
	}

	fclose(pFile);

	return dwSum;
}

void CodeHSEntry(SHSEntry* pEntry)
{
	BYTE* pByte;

	pByte = (BYTE*)(pEntry);
	for(int i = 0; i < sizeof(SHSEntry); i++) *(pByte++) ^= 0x5D;
}

bool IsValidHSEntry(SHSEntry* pEntry)
{
	if(pEntry->iSecurity1 != (int)(((pEntry->iMoney * 27 + (pEntry->iMoney % 163)) ^ 0x7FA9D018) + (strlen(pEntry->acName) * 45) + 13)) return false;
	if(fabsf(pEntry->fSecurity2 - (powf((float)(pEntry->iMoney), 1.15279827f) + 248.75319912f)) > 0.1f) return false;
	if(fabsf(pEntry->fSecurity3 - (sqrtf((float)(pEntry->iMoney)) * 1.24912629f)) > 0.1f) return false;
	return true;
}

void CalcSecurityValues(SHSEntry* pEntry)
{
	pEntry->iSecurity1 = (int)(((pEntry->iMoney * 27 + (pEntry->iMoney % 163)) ^ 0x7FA9D018) + (strlen(pEntry->acName) * 45) + 13);
	pEntry->fSecurity2 = powf((float)(pEntry->iMoney), 1.15279827f) + 248.75319912f;
	pEntry->fSecurity3 = sqrtf((float)(pEntry->iMoney)) * 1.24912629f;
}

DWORD TimeDateToValue(SYSTEMTIME* pTimeDate)
{
	return (DWORD)(pTimeDate->wSecond) +
	       (DWORD)(pTimeDate->wMinute) * 60 +
		   (DWORD)(pTimeDate->wHour) * 60 * 60 +
		   (DWORD)(pTimeDate->wDay) * 60 * 60 * 24 +
		   (DWORD)(pTimeDate->wMonth) * 60 * 60 * 24 * 31 +
		   (DWORD)(pTimeDate->wYear) * 60 * 60 * 24 * 31 * 12;
}

int HighScoreCmp(const void* pE1,
				 const void* pE2)
{
	SHSEntry* pEntry1;
	SHSEntry* pEntry2;


	pEntry1 = (SHSEntry*)(pE1);
	pEntry2 = (SHSEntry*)(pE2);
	if(pEntry1->iMoney != pEntry2->iMoney) return pEntry2->iMoney - pEntry1->iMoney;
	else return TimeDateToValue(&pEntry2->TimeDate) - TimeDateToValue(&pEntry1->TimeDate);
}

bool GetTop20(char* pcLevel,
			  SHSEntry* pOut21)
{
	DWORD	dwSum;
	FILE*	pFile;


	ZeroMemory(pOut21, 21 * sizeof(SHSEntry));
	dwSum = GetLevelCS(pcLevel);

	pFile = fopen("Data\\HighScores.hsc", "rb");
	if(pFile == NULL) return false;

	while(!feof(pFile))
	{
		if(fread(&pOut21[20], sizeof(SHSEntry), 1, pFile) != 1) break;
		CodeHSEntry(&pOut21[20]);
		if(pOut21[20].dwSum != dwSum || !IsValidHSEntry(&pOut21[20])) ZeroMemory(&pOut21[20], sizeof(SHSEntry));
		else
		{
			if(!strcmp(pOut21[20].acName, "Reset high scores") &&
			   pOut21[20].iMoney == -12345678)
			{
				ZeroMemory(pOut21, 21 * sizeof(SHSEntry));
			}
		}

		qsort(pOut21, 21, sizeof(SHSEntry), HighScoreCmp);
	}

	ZeroMemory(&pOut21[20], sizeof(SHSEntry));
	fclose(pFile);

	return true;
}

int GetTile(int x,
			int y)
{
	if(x < 0 || x >= g_iLevelWidth) return -2;
	else if(y < 0 || y >= g_iLevelHeight) return -1;
	return g_aiLevel[x][y];
}

int GetTileF(float x,
			 float y)
{
	if(x < 0.0f || x >= (float)(g_iLevelWidth) * 32.0f) return -2;
	else if(y < 0.0f || y >= (float)(g_iLevelHeight) * 32.0f) return -1;
	return g_aiLevel[(int)(x / 32.0f)][(int)(y / 32.0f)];
}

int GetTileType(int x,
				int y)
{
	int iTile;

	iTile = GetTile(x, y);
	if(iTile == -1) return 0;
	else if(iTile == -2) return 1;
	return g_aiTileType[iTile];
}

int GetTileTypeF(float x,
				 float y)
{
	int iTile;

	iTile = GetTileF(x, y);
	if(iTile == -1) return 0;
	else if(iTile == -2) return 1;
	else
	{
		float tx = (x / 32.0f) - (float)((int)(x) / 32);
		float ty = (y / 32.0f) - (float)((int)(y) / 32);

		if(g_aiTileType[iTile] == 4)
		{
			if(ty > 1.0f - tx * 0.5f) return 4;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 5)
		{
			if(ty > 0.5f - tx * 0.5f) return 5;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 6)
		{
			if(ty > 0.5f + tx * 0.5f) return 6;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 7)
		{
			if(ty > tx * 0.5f) return 7;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 8)
		{
			if(ty > 1.0f - tx) return 8;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 9)
		{
			if(ty > tx) return 9;
			else return 0;
		}
	}

	return g_aiTileType[iTile];
}

int GetTileTypeExF(float x,
				   float y)
{
	int			iTile;
	SElevator*	pElevator;


	for(int i = 0; i < g_iNumElevators; i++)
	{
		pElevator = &g_aElevator[i];
		if(x < (pElevator->x - (float)(pElevator->iWidth / 2))) continue;
		if(x > (pElevator->x + (float)(pElevator->iWidth / 2))) continue;
		if(y < (pElevator->y - (float)(pElevator->iHeight / 2))) continue;
		if(y > (pElevator->y + (float)(pElevator->iHeight / 2))) continue;
		return 100;
	}

	iTile = GetTileF(x, y);
	if(iTile == -1) return -1;
	else if(iTile == -2) return 1;
	else
	{
		float tx = (x / 32.0f) - (float)((int)(x) / 32);
		float ty = (y / 32.0f) - (float)((int)(y) / 32);

		if(g_aiTileType[iTile] == 4)
		{
			if(ty > 1.0f - tx * 0.5f) return 4;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 5)
		{
			if(ty > 0.5f - tx * 0.5f) return 5;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 6)
		{
			if(ty > 0.5f + tx * 0.5f) return 6;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 7)
		{
			if(ty > tx * 0.5f) return 7;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 8)
		{
			if(ty > 1.0f - tx) return 8;
			else return 0;
		}
		else if(g_aiTileType[iTile] == 9)
		{
			if(ty > tx) return 9;
			else return 0;
		}
	}

	return g_aiTileType[iTile];
}

void Delay(DWORD dwDuration)
{
	DWORD dwStartTime;

	dwStartTime = timeGetTime();
	while(timeGetTime() <= dwStartTime + dwDuration) {}
}

bool LoadLevel(char* pcFilename)
{
	FILE*		pFile;
	FILE*		pMapFile;
	FILE*		pTilesData;
	FILE*		pItemsData;
	char		acKey[256];
	char		acValue[256];
	char		acText[256];
	WORD		wTile;
	int			iTile;
	int			iType;
	int			iItem;
	int			iFrequency;
	int			iElevator;
	int			iPoint;
	float		x;
	float		y;
	SElevator	Temp;


	sprintf(acText, "Data\\%s", pcFilename);
	pFile = fopen(acText, "rb");
	if(pFile == NULL) return false;

	g_iNumElevators = 0;

	while(!feof(pFile))
	{
		fscanf(pFile, "%s = %s", acKey, acValue);
		if(!strcmp(acKey, "MapFile") ||
		   !strcmp(acKey, "TilesBmp") ||
		   !strcmp(acKey, "TilesData") ||
		   !strcmp(acKey, "ItemsBmp") ||
		   !strcmp(acKey, "ItemsData") ||
		   !strcmp(acKey, "MusicFile") ||
		   !strcmp(acKey, "Sky1") ||
		   !strcmp(acKey, "Sky2"))
		{
			strcpy(acText, acValue);
			sprintf(acValue, "Data\\%s", acText);
		}

		if(!strcmp(acKey, "MapSize")) sscanf(acValue, "%d,%d", &g_iLevelWidth, &g_iLevelHeight);
		else if(!strcmp(acKey, "MapFile"))
		{
			pMapFile = fopen(acValue, "rb");
			for(int y = 0; y < g_iLevelHeight; y++)
			{
				for(int x = 0; x < g_iLevelWidth; x++)
				{
					fread(&wTile, 1, sizeof(WORD), pMapFile);
					g_aiLevel[x][y] = (((wTile - 32) % 32) + (wTile - 32)) / 32;
				}
			}
			fclose(pMapFile);
		}
		else if(!strcmp(acKey, "TilesBmp")) LoadBitmap(acValue, &g_pTiles, 0);
		else if(!strcmp(acKey, "TilesData"))
		{
			pTilesData = fopen(acValue, "rb");
			iTile = 0;
			fscanf(pTilesData, "%d,", &g_iNumTilesPerRow);
			while(!feof(pTilesData)) {fscanf(pTilesData, "%d, ", &iType); g_aiTileType[iTile] = iType; iTile++;}
			fclose(pTilesData);
		}
		else if(!strcmp(acKey, "ItemsBmp")) LoadBitmap(acValue, &g_pItems, 0);
		else if(!strcmp(acKey, "ItemsData"))
		{
			pItemsData = fopen(acValue, "rb");
			iItem = 0;
			while(!feof(pItemsData))
			{
				fscanf(pItemsData, "%d, %d, %d, %d, %d, %d, %d, %f, %d, %d",
					   &g_aItemType[iItem].sy,
					   &g_aItemType[iItem].iValue,
					   &g_aItemType[iItem].iMagnet,
					   &g_aItemType[iItem].bRoulette,
					   &g_aItemType[iItem].iFlyingMode,
					   &g_aItemType[iItem].iNumAnimFrames,
					   &iFrequency,
					   &g_aItemType[iItem].fAnimSpeed,
					   &g_aItemType[iItem].iEx1,
					   &g_aItemType[iItem].iEx2);

				int iNum = 0;
				for(int i = 0; i < 100 && iNum < iFrequency; i++)
				{
					if(g_aiFreqTable[i] == -1)
					{
						g_aiFreqTable[i] = iItem;
						iNum++;
					}
				}

				iItem++;
			}

			fclose(pItemsData);
		}
		else if(!strcmp(acKey, "MusicFile")) strcpy(g_acMusic, acValue);
		else if(!strcmp(acKey, "TotalItems")) g_iTotalItems = atoi(acValue);
		else if(!strcmp(acKey, "Sky1")) LoadBitmap(acValue, &g_pSky1, 0);
		else if(!strcmp(acKey, "SkyZ1")) g_iSkyZ1 = atoi(acValue);
		else if(!strcmp(acKey, "Sky2")) LoadBitmap(acValue, &g_pSky2, 0);
		else if(!strcmp(acKey, "SkyZ2")) g_iSkyZ2 = atoi(acValue);
		else if(!strcmp(acKey, "Snow")) g_iSnow = atoi(acValue);
		else if(!strcmp(acKey, "SnowSprites")) sscanf(acValue, "%d,%d,%d,%d", &g_iSnowSpriteX, &g_iSnowSpriteY, &g_iSnowSpriteSize, &g_iNumSnowSprites);
		else if(!strcmp(acKey, "SnowDensity")) g_iSnowDensity = atoi(acValue);
		else if(!strcmp(acKey, "PlayerStart")) sscanf(acValue, "%f,%f", &g_Player.x, &g_Player.y);
		else if(!strcmp(acKey, "PlayerSpeed")) sscanf(acValue, "%f", &g_fPlayerSpeed);
		else if(!strcmp(acKey, "PlayerJumpFactor")) sscanf(acValue, "%f", &g_fPlayerJumpFactor);
		else if(!strcmp(acKey, "Gravity")) sscanf(acValue, "%f", &g_fGravity);
		else if(!strcmp(acKey, "Friction")) sscanf(acValue, "%f,%f", &g_fFrictionX, &g_fFrictionY);
		else if(!strcmp(acKey, "MinItemSpeed")) sscanf(acValue, "%f", &g_fMinItemSpeed);
		else if(!strcmp(acKey, "Time")) sscanf(acValue, "%f", &g_fCD);
		else if(!strcmp(acKey, "NumElevators")) g_iNumElevators = atoi(acValue);
		else if(!strcmp(acKey, "Elevator"))
		{
			sscanf(acValue, "%d,%d,%d,%d,%d,%f,%d,%d",
				   &iElevator,
				   &Temp.iSpriteX,
				   &Temp.iSpriteY,
				   &Temp.iWidth,
				   &Temp.iHeight,
				   &Temp.fSpeed,
				   &Temp.iNumPoints,
				   &Temp.iCurrentDest);
			Temp.fTimeLeft = 0.0f;
			g_aElevator[iElevator] = Temp;
		}
		else if(!strcmp(acKey, "ElevatorPoint"))
		{
			sscanf(acValue, "%d,%d,%f,%f",
				   &iElevator, &iPoint, &x, &y);
			g_aElevator[iElevator].afPointX[iPoint] = x;
			g_aElevator[iElevator].afPointY[iPoint] = y;
		}
	}

	fclose(pFile);

	return true;
}

void PlaySound(int iFirst,
			   int iLast = -1)
{
	int iSound;

	if(!g_Options.bUseSound) return;

	if(iLast == -1) iLast = iFirst;
	iSound = iFirst + (rand() % (iLast - iFirst + 1)) - 1;
	if(iSound != 21)
	{
		g_apSound[iSound]->SetCurrentPosition(0);
		g_apSound[iSound]->SetFrequency(44100 + (rand() % 10001) - 5000);
		g_apSound[iSound]->Play(0, 0, 0);
	}
	else g_apSound[iSound]->Play(0, 0, 0);
}

bool DrawLevel()
{
	if(g_iSnow != 2)
	{
		for(int y = (int)(g_fCameraY) / 32 - 0; y < ((int)(g_fCameraY) + 480) / 32 + 1; y++)
		{
			for(int x = (int)(g_fCameraX) / 32 - 0; x < ((int)(g_fCameraX) + 640) / 32 + 1; x++)
			{
				if(x >= 0 && x < g_iLevelWidth &&
				   y >= 0 && y < g_iLevelHeight)
				{
					if(g_aiLevel[x][y] != 0)
					{
						DrawSprite(g_pBackBuffer,
								   x * 32 - (int)(g_fCameraX),
								   y * 32 - (int)(g_fCameraY),
								   32, 32,
 								   g_pTiles,
								   GetTileX(g_aiLevel[x][y]),
								   GetTileY(g_aiLevel[x][y]),
								   32, 32,
								   true);
					}
				}
			}
		}
	}
	else
	{
		for(int y = (int)(g_fCameraY) / 32 - 0; y < ((int)(g_fCameraY) + 480) / 32 + 1; y++)
		{
			float fx = sinf(2.0f * g_fCD + (float)(y) * 0.25f) * 5.0f;
			for(int x = (int)(g_fCameraX) / 32 - 0; x < ((int)(g_fCameraX) + 640) / 32 + 1; x++)
			{
				if(x >= 0 && x < g_iLevelWidth &&
				   y >= 0 && y < g_iLevelHeight)
				{
					if(g_aiLevel[x][y] != 0)
					{
						DrawSprite(g_pBackBuffer,
								   x * 32 - (int)(g_fCameraX) - (int)(fx),
								   y * 32 - (int)(g_fCameraY),
								   32, 32,
 								   g_pTiles,
								   GetTileX(g_aiLevel[x][y]),
								   GetTileY(g_aiLevel[x][y]),
								   32, 32,
								   true);
					}
				}
			}
		}
	}

	return true;
}

void DrawSky()
{
	if(g_pSky2 != NULL)
	{
		Blt(g_pBackBuffer, 0, 0, 640, 480,
			g_pSky2, (int)(g_fCameraX) / g_iSkyZ2, (int)(g_fCameraY) / g_iSkyZ2, 640, 480,
			0,
			NULL);
	}

	if(g_pSky1 != NULL)
	{
		Blt(g_pBackBuffer, 0, 0, 640, 480,
			g_pSky1, (int)(g_fCameraX) / g_iSkyZ1, (int)(g_fCameraY) / g_iSkyZ1, 640, 480,
			DDBLT_KEYSRC,
			NULL);
	}
}

BOOL CALLBACK SetCursorOfChildrenCallback(HWND hChild,
										  LPARAM LParam)
{
	SetClassLong(hChild, GCL_HCURSOR, LParam);
	return TRUE;
}

void SetCursorOfChildren(HWND hParent,
						 HCURSOR hCursor)
{
	EnumChildWindows(hParent, SetCursorOfChildrenCallback, (LPARAM)(hCursor));
}

BOOL CALLBACK OptionsDlgProc(HWND hDlg,
							 unsigned int uiMsg,
							 WPARAM WParam,
							 LPARAM LParam)
{
	FILE* pFile;

	switch(uiMsg)
	{
	case WM_INITDIALOG:
		SetClassLong(hDlg, GCL_HICON, (LONG)(LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON1))));
#ifndef NTD
		SetWindowLong(hDlg, GWL_EXSTYLE, GetWindowLong(hDlg, GWL_EXSTYLE) | WS_EX_LAYERED);
		SetLayeredWindowAttributes(hDlg, 0, 210, LWA_ALPHA);
#endif
		SetClassLong(hDlg, GCL_HCURSOR, (LONG)(g_hCursor));
		SetCursorOfChildren(hDlg, g_hCursor);
		CheckDlgButton(hDlg, IDC_O_SKY, g_Options.bShowSky);
		CheckDlgButton(hDlg, IDC_O_SNOW, g_Options.bShowSnow);
		CheckDlgButton(hDlg, IDC_O_SOUND, g_Options.bUseSound);
		CheckDlgButton(hDlg, IDC_O_MUSIC, g_Options.bUseMusic);
		CheckDlgButton(hDlg, IDC_O_NOHIPRIO, g_Options.bNoHiPrio);
		break;

	case WM_SETCURSOR:
		SetCursor(g_hCursor);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, 0);
		break;

	case WM_COMMAND:
		switch(LOWORD(WParam))
		{
		case IDC_O_OK:
			g_Options.bShowSky = IsDlgButtonChecked(hDlg, IDC_O_SKY);
			g_Options.bShowSnow = IsDlgButtonChecked(hDlg, IDC_O_SNOW);
			g_Options.bUseSound = IsDlgButtonChecked(hDlg, IDC_O_SOUND);
			g_Options.bUseMusic = IsDlgButtonChecked(hDlg, IDC_O_MUSIC);
			g_Options.bNoHiPrio = IsDlgButtonChecked(hDlg, IDC_O_NOHIPRIO);
			pFile = fopen("Data\\Ricardo2.cfg", "wb");
			fwrite(&g_Options, sizeof(SOptions), 1, pFile);
			fclose(pFile);
			EndDialog(hDlg, 0);
			break;

		case IDC_O_CANCEL:
			EndDialog(hDlg, 0);
			break;
		}
		break;

	default:
		return FALSE;
		break;
	}

	return TRUE;
}

BOOL CALLBACK HelpDlgProc(HWND hDlg,
						  unsigned int uiMsg,
						  WPARAM WParam,
						  LPARAM LParam)
{
	switch(uiMsg)
	{
	case WM_INITDIALOG:
		SetClassLong(hDlg, GCL_HICON, (LONG)(LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON1))));
#ifndef NTD
		SetWindowLong(hDlg, GWL_EXSTYLE, GetWindowLong(hDlg, GWL_EXSTYLE) | WS_EX_LAYERED);
		SetLayeredWindowAttributes(hDlg, 0, 210, LWA_ALPHA);
#endif
		break;

	case WM_SETCURSOR:
		SetCursor(g_hCursor);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, 0);
		break;

	case WM_COMMAND:
		switch(LOWORD(WParam))
		{
		case IDC_HLP_BACK:
			EndDialog(hDlg, 0);
			break;
		}
		break;

	default:
		return FALSE;
		break;
	}

	return TRUE;
}

BOOL CALLBACK MainMenuDlgProc(HWND hDlg,
							  unsigned int uiMsg,
							  WPARAM WParam,
							  LPARAM LParam)
{
	char acText[256];

	switch(uiMsg)
	{
	case WM_INITDIALOG:
		GetDlgItemText(hDlg, IDC_MM_PLAY, acText, 256);
		g_bEnglish = strcmp(acText, "&Spiel beginnen");
		SetClassLong(hDlg, GCL_HICON, (LONG)(LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON1))));
#ifndef NTD
		SetWindowLong(hDlg, GWL_EXSTYLE, GetWindowLong(hDlg, GWL_EXSTYLE) | WS_EX_LAYERED);
		SetLayeredWindowAttributes(hDlg, 0, 210, LWA_ALPHA);
#endif
		SetClassLong(hDlg, GCL_HCURSOR, (LONG)(g_hCursor));
		SetCursorOfChildren(hDlg, g_hCursor);
		break;

	case WM_SETCURSOR:
		SetCursor(g_hCursor);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, 0);
		break;

	case WM_COMMAND:
		switch(LOWORD(WParam))
		{
		case IDC_MM_PLAY:
			EndDialog(hDlg, 1);
			break;

		case IDC_MM_HELP:
			EndDialog(hDlg, 2);
			break;

		case IDC_MM_HIGHSCORES:
			EndDialog(hDlg, 3);
			break;

		case IDC_MM_ONLINEHIGHSCORES:
			EndDialog(hDlg, 4);
			break;

		case IDC_MM_OPTIONS:
			EndDialog(hDlg, 5);
			break;

		case IDC_MM_QUIT:
			EndDialog(hDlg, 6);
			break;
		}
		break;

	default:
		return FALSE;
		break;
	}

	return TRUE;
}

BOOL CALLBACK SelectLevelDlgProc(HWND hDlg,
								 unsigned int uiMsg,
								 WPARAM WParam,
								 LPARAM LParam)
{
	struct _finddata_t	File;
	long				lFile;
	int					iLevel;
	char				acLevel[256];


	switch(uiMsg)
	{
	case WM_INITDIALOG:
		SetClassLong(hDlg, GCL_HICON, (LONG)(LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON1))));
#ifndef NTD
		SetWindowLong(hDlg, GWL_EXSTYLE, GetWindowLong(hDlg, GWL_EXSTYLE) | WS_EX_LAYERED);
		SetLayeredWindowAttributes(hDlg, 0, 210, LWA_ALPHA);
#endif
		SetClassLong(hDlg, GCL_HCURSOR, (LONG)(g_hCursor));
		SetCursorOfChildren(hDlg, g_hCursor);
		lFile = _findfirst("Data\\*.lvl", &File);
		while(true)
		{
			SendDlgItemMessage(hDlg, IDC_SL_LEVELS, LB_ADDSTRING, 0, (LPARAM)(File.name));
			if(_findnext(lFile, &File) == -1) break;
		}

		_findclose(lFile);
		break;

	case WM_SETCURSOR:
		SetCursor(g_hCursor);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, 0);
		break;

	case WM_COMMAND:
		switch(LOWORD(WParam))
		{
		case IDC_SL_PLAY:
			iLevel = (int)(SendDlgItemMessage(hDlg, IDC_SL_LEVELS, LB_GETCURSEL, 0, 0));
			if(iLevel == CB_ERR)
			{
				if(!g_bEnglish) MessageBox(hDlg, "Whlen Sie zuerst einen Level aus!", "Fehler", MB_OK | MB_ICONEXCLAMATION);
				else MessageBox(hDlg, "Select a level first!", "Error", MB_OK | MB_ICONEXCLAMATION);
			}
			else
			{
				SendDlgItemMessage(hDlg, IDC_SL_LEVELS, LB_GETTEXT, iLevel, (LPARAM)(acLevel));
				g_iLevel = iLevel;
				strcpy(g_acLevel, acLevel);
				EndDialog(hDlg, 1);
			}
			break;

		case IDC_SL_BACK:
			EndDialog(hDlg, 2);
			break;
		}
		break;

	default:
		return FALSE;
		break;
	}

	return TRUE;
}

void UpdateHighScores(HWND hDlg,
					  int iListID,
					  int iLevelID)
{
	char		acLevel[256];
	SHSEntry	aTop20[21];
	char		acText[1024];


	SendDlgItemMessage(hDlg, iListID, LB_RESETCONTENT, 0, 0);

	GetDlgItemText(hDlg, iLevelID, acLevel, 256);

	GetTop20(acLevel, aTop20);
	for(int i = 0; i < 20; i++)
	{
		if(aTop20[i].iMoney > 0)
		{
			sprintf(acText, "%d.\t%d\t%s\t%d. %d. %d, %0.2d:%0.2d:%0.2d",
					i + 1,
					aTop20[i].iMoney,
					aTop20[i].acName,
					aTop20[i].TimeDate.wDay,
					aTop20[i].TimeDate.wMonth,
					aTop20[i].TimeDate.wYear,
					aTop20[i].TimeDate.wHour,
					aTop20[i].TimeDate.wMinute,
					aTop20[i].TimeDate.wSecond);
			SendDlgItemMessage(hDlg, iListID, LB_ADDSTRING, 0, (LPARAM)(acText));
		}
	}

	if(aTop20[0].iMoney == 0)
	{
		if(!g_bEnglish) SendDlgItemMessage(hDlg, iListID, LB_ADDSTRING, 0, (LPARAM)("Fr diesen Level gibt es noch keine High-Scores!"));
		else SendDlgItemMessage(hDlg, iListID, LB_ADDSTRING, 0, (LPARAM)("There are no high scores for that level yet."));
	}
}

bool AddHighScore(char* pcName,
				  int iMoney,
				  char* pcLevelFile)
{
	FILE*		pFile;
	SHSEntry	Entry;


	pFile = fopen("Data\\HighScores.hsc", "ab");
	if(pFile == NULL) return false;

	Entry.dwSum = GetLevelCS(pcLevelFile);
	strcpy(Entry.acName, pcName);
	Entry.iMoney = iMoney;
	GetLocalTime(&Entry.TimeDate);
	CalcSecurityValues(&Entry);
	CodeHSEntry(&Entry);
	fwrite(&Entry, sizeof(SHSEntry), 1, pFile);

	fclose(pFile);

	return true;
}

BOOL CALLBACK HighScoresDlgProc(HWND hDlg,
								unsigned int uiMsg,
								WPARAM WParam,
								LPARAM LParam)
{
	struct _finddata_t	File;
	long				lFile;
	int					iIndex;
	SHSEntry			aTop21[21];
	char				acLevel[256];
	char				acName[256];
	int					aiTabStop[3] = {15, 45, 115};


	switch(uiMsg)
	{
	case WM_INITDIALOG:
		SetClassLong(hDlg, GCL_HICON, (LONG)(LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON1))));
#ifndef NTD
		SetWindowLong(hDlg, GWL_EXSTYLE, GetWindowLong(hDlg, GWL_EXSTYLE) | WS_EX_LAYERED);
		SetLayeredWindowAttributes(hDlg, 0, 210, LWA_ALPHA);
#endif
		SetClassLong(hDlg, GCL_HCURSOR, (LONG)(g_hCursor));
		SetCursorOfChildren(hDlg, g_hCursor);

		if(LParam)
		{
			if(!g_bEnglish) SetDlgItemText(hDlg, IDC_HS_PLAYAGAIN, "Level &erneut spielen");
			else SetDlgItemText(hDlg, IDC_HS_PLAYAGAIN, "Play &again");
			if(!g_bEnglish) sprintf(acName, "High-Scores - Sie haben %d Punkte erreicht!", g_Player.iMoney);
			else sprintf(acName, "High scores - you have got %d points!", g_Player.iMoney);
			SetWindowText(hDlg, acName);
		}
		else
		{
			if(!g_bEnglish) SetDlgItemText(hDlg, IDC_HS_PLAYAGAIN, "Diesen Level &spielen");
			else SetDlgItemText(hDlg, IDC_HS_PLAYAGAIN, "&Play that level");
		}

		SendDlgItemMessage(hDlg, IDC_HS_LIST, LB_SETTABSTOPS, 3, (LPARAM)(aiTabStop));
		EnableWindow(GetDlgItem(hDlg, IDC_HS_NAMELABEL), FALSE);
		EnableWindow(GetDlgItem(hDlg, IDC_HS_NAME), FALSE);
		EnableWindow(GetDlgItem(hDlg, IDC_HS_OK), FALSE);

		lFile = _findfirst("Data\\*.lvl", &File);
		while(true)
		{
			iIndex = SendDlgItemMessage(hDlg, IDC_HS_LEVELS, CB_ADDSTRING, 0, (LPARAM)(File.name));
			if(_findnext(lFile, &File) == -1) break;
		}

		if(!LParam) SendDlgItemMessage(hDlg, IDC_HS_LEVELS, CB_SETCURSEL, 0, 0);
		else SendDlgItemMessage(hDlg, IDC_HS_LEVELS, CB_SETCURSEL, g_iLevel, 0);
		SetTimer(hDlg, LParam, 100, NULL);
		break;

	case WM_SETCURSOR:
		SetCursor(g_hCursor);
		break;

	case WM_TIMER:
		KillTimer(hDlg, WParam);

		if(WParam == 1)
		{
			SendDlgItemMessage(hDlg, IDC_HS_LEVELS, CB_SETCURSEL, g_iLevel, 0);
			GetDlgItemText(hDlg, IDC_HS_LEVELS, acLevel, 256);
			GetTop20(acLevel, aTop21);
			if(g_Player.iMoney >= aTop21[19].iMoney &&
			   g_Player.iMoney > 0)
			{
				EnableWindow(GetDlgItem(hDlg, IDC_HS_NAMELABEL), TRUE);
				EnableWindow(GetDlgItem(hDlg, IDC_HS_NAME), TRUE);
				EnableWindow(GetDlgItem(hDlg, IDC_HS_OK), TRUE);
				if(!g_bEnglish) SetDlgItemText(hDlg, IDC_HS_NOTIFICATION, "Sie haben einen Platz in den High-Scores erzielt! Bitte geben Sie unten Ihren Namen ein und drcken Sie [Return] oder klicken Sie auf \"OK\"!");
				else SetDlgItemText(hDlg, IDC_HS_NOTIFICATION, "You achieved a place in the high scores! Please enter your name below and press [Return] or click \"OK\"!");
				SetFocus(GetDlgItem(hDlg, IDC_HS_NAME));
			}
		}

		UpdateHighScores(hDlg, IDC_HS_LIST, IDC_HS_LEVELS);
		break;

	case WM_CLOSE:
		if(IsWindowEnabled(GetDlgItem(hDlg, IDC_HS_NAME)))
		{
			if(MessageBox(hDlg,
			   g_bEnglish ? "You achieved a place in the high scores, but you haven't entered you name yet.\nAre you sure you want to continue?" : "Sie haben einen Platz in den High-Scores erreicht, aber Ihren Namen noch nicht eingegeben!\nSind Sie sicher, dass Sie fortfahren mchten?",
			   g_bEnglish ? "Confirm" : "Fortfahren besttigen", MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) == IDYES)
			{
				EndDialog(hDlg, 0);
			}
			else SetFocus(GetDlgItem(hDlg, IDC_HS_NAME));
		}
		else EndDialog(hDlg, 0);
		break;

	case WM_COMMAND:
		switch(LOWORD(WParam))
		{
		case IDC_HS_LEVELS:
			if(HIWORD(WParam) == CBN_SELCHANGE)
			{
				if(IsWindowEnabled(GetDlgItem(hDlg, IDC_HS_OK)) &&
				   SendDlgItemMessage(hDlg, IDC_HS_LEVELS, CB_GETCURSEL, 0, 0) != g_iLevel)
				{
					if(MessageBox(hDlg,
					   g_bEnglish ? "You achieved a place in the high scores, but you haven't entered you name yet.\nAre you sure you want to continue?" : "Sie haben einen Platz in den High-Scores erreicht, aber Ihren Namen noch nicht eingegeben!\nSind Sie sicher, dass Sie fortfahren mchten?",
					   g_bEnglish ? "Confirm" : "Fortfahren besttigen", MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) == IDYES)
					{
						UpdateHighScores(hDlg, IDC_HS_LIST, IDC_HS_LEVELS);
						SetDlgItemText(hDlg, IDC_HS_PLAYAGAIN, g_bEnglish ? "&Play that level" : "Diesen Level &spielen");
						SetDlgItemText(hDlg, IDC_HS_NAME, "");
						SetDlgItemText(hDlg, IDC_HS_NOTIFICATION, "");
						EnableWindow(GetDlgItem(hDlg, IDC_HS_NAMELABEL), FALSE);
						EnableWindow(GetDlgItem(hDlg, IDC_HS_NAME), FALSE);
						EnableWindow(GetDlgItem(hDlg, IDC_HS_OK), FALSE);
					}
					else
					{
						SendDlgItemMessage(hDlg, IDC_HS_LEVELS, CB_SETCURSEL, g_iLevel, 0);
					}
				}
				else
				{
					if(SendDlgItemMessage(hDlg, IDC_HS_LEVELS, CB_GETCURSEL, 0, 0) != g_iLevel)
					{
						SetDlgItemText(hDlg, IDC_HS_PLAYAGAIN, g_bEnglish ? "&Play that level" : "Diesen Level &spielen");
						UpdateHighScores(hDlg, IDC_HS_LIST, IDC_HS_LEVELS);
					}
				}
			}
			break;

		case IDC_HS_PLAYAGAIN:
			strcpy(acLevel, "");
			GetDlgItemText(hDlg, IDC_HS_LEVELS, acLevel, 256);
			if(strlen(acLevel) <= 0)
			{
				MessageBox(hDlg, g_bEnglish ? "Select a level first!" : "Whlen Sie zuerst einen Level aus!",
						   g_bEnglish ? "Fehler" : "Error",
					       MB_OK | MB_ICONEXCLAMATION);
			}
			else
			{
				if(IsWindowEnabled(GetDlgItem(hDlg, IDC_HS_NAME)))
				{
					if(MessageBox(hDlg,
					   g_bEnglish ? "You achieved a place in the high scores, but you haven't entered you name yet.\nAre you sure you want to continue?" : "Sie haben einen Platz in den High-Scores erreicht, aber Ihren Namen noch nicht eingegeben!\nSind Sie sicher, dass Sie fortfahren mchten?",
					   g_bEnglish ? "Confirm" : "Fortfahren besttigen", MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) == IDYES)
					{
						g_iLevel = SendDlgItemMessage(hDlg, IDC_HS_LEVELS, CB_GETCURSEL, 0, 0);
						strcpy(g_acLevel, acLevel);
						EndDialog(hDlg, 1);
					}
					else SetFocus(GetDlgItem(hDlg, IDC_HS_NAME));
				}
				else
				{
					g_iLevel = SendDlgItemMessage(hDlg, IDC_HS_LEVELS, CB_GETCURSEL, 0, 0);
					strcpy(g_acLevel, acLevel);
					EndDialog(hDlg, 1);
				}
			}
			break;

		case IDC_HS_BACK:
			if(IsWindowEnabled(GetDlgItem(hDlg, IDC_HS_NAME)))
			{
				if(MessageBox(hDlg,
				   g_bEnglish ? "You achieved a place in the high scores, but you haven't entered you name yet.\nAre you sure you want to continue?" : "Sie haben einen Platz in den High-Scores erreicht, aber Ihren Namen noch nicht eingegeben!\nSind Sie sicher, dass Sie fortfahren mchten?",
				   g_bEnglish ? "Confirm" : "Fortfahren besttigen", MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) == IDYES)
				{
					EndDialog(hDlg, 2);
				}
				else SetFocus(GetDlgItem(hDlg, IDC_HS_NAME));
			}
			else EndDialog(hDlg, 2);
			break;

		case IDC_HS_RESET:
			if(MessageBox(hDlg, g_bEnglish ? "Are you sure that you want the high scores of that level to be reset?" : "Sind Sie sicher, dass Sie die High-Scores dieses Levels zurcksetzen mchten?",
				          g_bEnglish ? "Confirm" : "Zurcksetzen besttigen", MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) == IDYES)
			{
				FILE*		pFile;
				SHSEntry	Entry;


				pFile = fopen("Data\\HighScores.hsc", "ab");
				
				GetDlgItemText(hDlg, IDC_HS_LEVELS, acLevel, 256);
				Entry.dwSum = GetLevelCS(acLevel);
				strcpy(Entry.acName, "Reset high scores");
				Entry.iMoney = -12345678;
				CalcSecurityValues(&Entry);
				CodeHSEntry(&Entry);
				fwrite(&Entry, sizeof(SHSEntry), 1, pFile);

				fclose(pFile);

				UpdateHighScores(hDlg, IDC_HS_LIST, IDC_HS_LEVELS);
			}
			break;

		case IDC_HS_RESETALL:
			if(MessageBox(hDlg, g_bEnglish ? "Are you sure that you want the high scores of all levels to be reset?" : "Sind Sie sicher, dass Sie die High-Scores aller Levels zurcksetzen mchten?",
				          g_bEnglish ? "Confirm" : "Zurcksetzen besttigen", MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) == IDYES)
			{
				DeleteFile("Data\\HighScores.hsc");
				UpdateHighScores(hDlg, IDC_HS_LIST, IDC_HS_LEVELS);
			}
			break;

		case IDC_HS_OK:
			GetDlgItemText(hDlg, IDC_HS_NAME, acName, 256);
			if(strlen(acName) == 0)
			{
				MessageBox(hDlg, g_bEnglish ? "Please enter your name!" : "Bitte geben Sie Ihren Namen ein!",
					       g_bEnglish ? "Error" : "Fehler",
				           MB_OK | MB_ICONEXCLAMATION);
				break;
			}

			GetDlgItemText(hDlg, IDC_HS_LEVELS, acLevel, 256);
			AddHighScore(acName, g_Player.iMoney, acLevel);

			SetDlgItemText(hDlg, IDC_HS_NAME, "");
			EnableWindow(GetDlgItem(hDlg, IDC_HS_NAMELABEL), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_HS_NAME), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_HS_OK), FALSE);
			SetDlgItemText(hDlg, IDC_HS_NOTIFICATION, "");
			UpdateHighScores(hDlg, IDC_HS_LIST, IDC_HS_LEVELS);
			break;
		}
		break;

	default:
		return FALSE;
		break;
	}

	return TRUE;
}

bool GetOnlineTop20(DWORD dwSum,
					SHSEntry* pOut)
{
	HINTERNET	hInternet;
	HINTERNET	hFile;
	char		acFile[256];
	DWORD		dwNumBytesRead;
	char		acBuffer[16384] = "Leer";
	char*		pcCursor;


	ZeroMemory(pOut, 20 * sizeof(SHSEntry));

	if(InternetAttemptConnect(0) != ERROR_SUCCESS) return false;
	if(!InternetCheckConnection("http://www.scherfgen-software.de", FLAG_ICC_FORCE_CONNECTION, 0)) return false;

	hInternet = InternetOpen("Scherfgen-Software Ricardo 2",
		                     INTERNET_OPEN_TYPE_PRECONFIG,
							 NULL,
							 NULL,
							 0);
	if(hInternet == NULL) return false;

	sprintf(acFile, "http://www.scherfgen-software.de/ricardo2/get_highscores.php?sum=%d", dwSum);
	hFile = InternetOpenUrl(hInternet,
		                    acFile,
							NULL,
							0,
							INTERNET_FLAG_RELOAD,
							0);
	if(hFile == NULL)
	{
		InternetCloseHandle(hInternet);
		return false;
	}

	ZeroMemory(acBuffer, 16384);
	if(!InternetReadFile(hFile,
						 acBuffer,
						 16384,
						 &dwNumBytesRead))
	{
		InternetCloseHandle(hFile);
		InternetCloseHandle(hInternet);
		return false;
	}

	if(strncmp(acBuffer, "Leer", 4))
	{
		acBuffer[dwNumBytesRead] = 0;
		pcCursor = acBuffer;

		for(int i = 0; i < 20; i++)
		{
			sscanf(pcCursor, "%s\t%hd\t%hd\t%hd\t%hd\t%hd\t%hd\t%hd",
				   pOut[i].acName,
				   &pOut[i].TimeDate.wDay,
				   &pOut[i].TimeDate.wMonth,
				   &pOut[i].TimeDate.wYear,
				   &pOut[i].TimeDate.wHour,
				   &pOut[i].TimeDate.wMinute,
				   &pOut[i].TimeDate.wSecond,
				   &pOut[i].iMoney);
			if(i != 19)
			{
				pcCursor++;
				while(*pcCursor != '\n') pcCursor++;
			}

			for(int j = 0; j < (int)(strlen(pOut[i].acName)); j++) if(pOut[i].acName[j] == '|') pOut[i].acName[j] = ' ';
		}
	}

	InternetCloseHandle(hFile);
	InternetCloseHandle(hInternet);

	return true;
}

void GetOnlineHighScores(HWND hDlg,
						 int iListID,
						 int iLevelID)
{
	char		acLevel[256];
	SHSEntry	aLocalTop20[21];
	SHSEntry	aTop20[20];
	DWORD		dwSum;
	char		acText[1024];


	SetCursor(g_hWaitCursor);
	Delay(100);

	SendDlgItemMessage(hDlg, iListID, LB_RESETCONTENT, 0, 0);

	GetDlgItemText(hDlg, iLevelID, acLevel, 256);
	dwSum = GetLevelCS(acLevel);

	GetTop20(acLevel, aLocalTop20);

	if(!GetOnlineTop20(dwSum, aTop20))
	{
		SendDlgItemMessage(hDlg, iListID, LB_ADDSTRING, 0,
			               (LPARAM)(g_bEnglish ? "Error - connection could not be established" : "Fehler - konnte keine Verbindung herstellen!"));
		return;
	}

	EnableWindow(GetDlgItem(hDlg, IDC_OHS_UPLOAD), FALSE);
	for(int i = 0; i < 20; i++)
	{
		if(aTop20[i].iMoney > 0)
		{
			sprintf(acText, "%d.\t%d\t%s\t%d. %d. %d, %0.2d:%0.2d:%0.2d",
					i + 1,
					aTop20[i].iMoney,
					aTop20[i].acName,
					aTop20[i].TimeDate.wDay,
					aTop20[i].TimeDate.wMonth,
					aTop20[i].TimeDate.wYear,
					aTop20[i].TimeDate.wHour,
					aTop20[i].TimeDate.wMinute,
					aTop20[i].TimeDate.wSecond);
			SendDlgItemMessage(hDlg, iListID, LB_ADDSTRING, 0, (LPARAM)(acText));
		}
	}

	for(int j = 0; j < 20; j++)
	{
		if(aLocalTop20[j].iMoney > aTop20[19].iMoney ||
		   (aLocalTop20[j].iMoney == aTop20[19].iMoney &&
			TimeDateToValue(&aLocalTop20[j].TimeDate) > TimeDateToValue(&aTop20[19].TimeDate)))
		{
			EnableWindow(GetDlgItem(hDlg, IDC_OHS_UPLOAD), TRUE);
		}
	}

	if(aTop20[0].iMoney == 0)
	{
		SendDlgItemMessage(hDlg, iListID, LB_ADDSTRING, 0,
			               (LPARAM)(g_bEnglish ? "There are no high scores for that level yet." : "Fr diesen Level gibt es noch keine High-Scores!"));
	}
}

bool UploadHighScores(char* pcLevelFile)
{
	SHSEntry	aLocalTop20[21];
	HINTERNET	hInternet;
	HINTERNET	hFile;
	char		acFile[1024];
	BYTE		aBuffer[256];
	DWORD		dwNumBytesRead;
	DWORD		dwSum;


	SetCursor(g_hWaitCursor);
	Delay(100);

	GetTop20(pcLevelFile, aLocalTop20);
	dwSum = GetLevelCS(pcLevelFile);

	if(InternetAttemptConnect(0) != ERROR_SUCCESS) return false;
	if(!InternetCheckConnection("http://www.scherfgen-software.de", FLAG_ICC_FORCE_CONNECTION, 0)) return false;

	hInternet = InternetOpen("Scherfgen-Software Ricardo 2",
		                     INTERNET_OPEN_TYPE_PRECONFIG,
							 NULL,
							 NULL,
							 0);
	if(hInternet == NULL) return false;

	for(int i = 0; i < 20; i++)
	{
		if(aLocalTop20[i].iMoney <= 0) continue;

		for(int j = 0; j < (int)(strlen(aLocalTop20[i].acName)); j++)
		{
			if(aLocalTop20[i].acName[j] == '\t') aLocalTop20[i].acName[j] = ' ';
			else if(aLocalTop20[i].acName[j] == '\n') aLocalTop20[i].acName[j] = ' ';
			else if(aLocalTop20[i].acName[j] == 0) aLocalTop20[i].acName[j] = ' ';
			else if(aLocalTop20[i].acName[j] == '|') aLocalTop20[i].acName[j] = 'I';
			else if(aLocalTop20[i].acName[j] == ' ') aLocalTop20[i].acName[j] = '|';
		}

		sprintf(acFile, "http://www.scherfgen-software.de/ricardo2/add_highscore.php?sum=%d&name=%s&day=%d&month=%d&year=%d&hour=%d&minute=%d&second=%d&money=%d&security1=%d&security2=%f&security3=%f",
			    dwSum,
				aLocalTop20[i].acName,
				aLocalTop20[i].TimeDate.wDay,
				aLocalTop20[i].TimeDate.wMonth,
				aLocalTop20[i].TimeDate.wYear,
				aLocalTop20[i].TimeDate.wHour,
				aLocalTop20[i].TimeDate.wMinute,
				aLocalTop20[i].TimeDate.wSecond,
				aLocalTop20[i].iMoney,
				aLocalTop20[i].iSecurity1,
				aLocalTop20[i].fSecurity2,
				aLocalTop20[i].fSecurity3);

		hFile = InternetOpenUrl(hInternet,
								acFile,
								NULL,
								0,
								INTERNET_FLAG_RELOAD,
								0);
		if(hFile == NULL)
		{
			InternetCloseHandle(hInternet);
			return false;
		}

		InternetReadFile(hFile,
						 aBuffer,
						 256,
						 &dwNumBytesRead);

		InternetCloseHandle(hFile);
	}

	InternetCloseHandle(hInternet);

	return true;
}

BOOL CALLBACK OnlineHighScoresDlgProc(HWND hDlg,
									  unsigned int uiMsg,
									  WPARAM WParam,
									  LPARAM LParam)
{
	struct _finddata_t	File;
	long				lFile;
	char				acLevel[256];
	int					aiTabStop[3] = {15, 45, 105};


	switch(uiMsg)
	{
	case WM_INITDIALOG:
		SetClassLong(hDlg, GCL_HICON, (LONG)(LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON1))));
#ifndef NTD
		SetWindowLong(hDlg, GWL_EXSTYLE, GetWindowLong(hDlg, GWL_EXSTYLE) | WS_EX_LAYERED);
		SetLayeredWindowAttributes(hDlg, 0, 210, LWA_ALPHA);
#endif
		SetClassLong(hDlg, GCL_HCURSOR, (LONG)(g_hCursor));
		SetCursorOfChildren(hDlg, g_hCursor);

		SendDlgItemMessage(hDlg, IDC_OHS_ONLINE, LB_SETTABSTOPS, 3, (LPARAM)(aiTabStop));
		SendDlgItemMessage(hDlg, IDC_OHS_LOCAL, LB_SETTABSTOPS, 3, (LPARAM)(aiTabStop));

		lFile = _findfirst("Data\\*.lvl", &File);
		while(true)
		{
			SendDlgItemMessage(hDlg, IDC_OHS_LEVELS, CB_ADDSTRING, 0, (LPARAM)(File.name));
			if(_findnext(lFile, &File) == -1) break;
		}

		SendDlgItemMessage(hDlg, IDC_OHS_LEVELS, CB_SETCURSEL, 0, 0);
		SetTimer(hDlg, 0, 100, NULL);
		break;

	case WM_TIMER:
		KillTimer(hDlg, 0);
		UpdateHighScores(hDlg, IDC_OHS_LOCAL, IDC_OHS_LEVELS);
		GetOnlineHighScores(hDlg, IDC_OHS_ONLINE, IDC_OHS_LEVELS);
		break;

	case WM_SETCURSOR:
		SetCursor(g_hCursor);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, 0);
		break;

	case WM_COMMAND:
		switch(LOWORD(WParam))
		{
		case IDC_OHS_LEVELS:
			if(HIWORD(WParam) == CBN_SELCHANGE)
			{
				UpdateHighScores(hDlg, IDC_OHS_LOCAL, IDC_OHS_LEVELS);
				GetOnlineHighScores(hDlg, IDC_OHS_ONLINE, IDC_OHS_LEVELS);
			}
			break;

		case IDC_OHS_PLAY:
			strcpy(acLevel, "");
			GetDlgItemText(hDlg, IDC_OHS_LEVELS, acLevel, 256);
			if(strlen(acLevel) <= 0)
			{
				MessageBox(hDlg, g_bEnglish ? "Select a level first!" : "Whlen Sie zuerst einen Level aus!",
					       g_bEnglish ? "Error" : "Fehler",
					       MB_OK | MB_ICONEXCLAMATION);
			}
			else
			{
				g_iLevel = SendDlgItemMessage(hDlg, IDC_OHS_LEVELS, CB_GETCURSEL, 0, 0);
				strcpy(g_acLevel, acLevel);
				EndDialog(hDlg, 1);
			}
			break;

		case IDC_OHS_UPDATE:
			UpdateHighScores(hDlg, IDC_OHS_LOCAL, IDC_OHS_LEVELS);
			GetOnlineHighScores(hDlg, IDC_OHS_ONLINE, IDC_OHS_LEVELS);
			break;

		case IDC_OHS_UPLOAD:
			GetDlgItemText(hDlg, IDC_OHS_LEVELS, acLevel, 256);
			UploadHighScores(acLevel);
			GetOnlineHighScores(hDlg, IDC_OHS_ONLINE, IDC_OHS_LEVELS);
			break;

		case IDC_OHS_BACK:
			EndDialog(hDlg, 2);
			break;
		}
		break;

	default:
		return FALSE;
		break;
	}

	return TRUE;
}

void MovePlayer(float fTime)
{
	SPlayer			OldPlayer;
	float			fOldVX;
	float			fFrictionX = g_fFrictionX;
	static float	fOuchCounter = 0.0f;
	static BOOL		bAahSoundPlaying = FALSE;


	OldPlayer = g_Player;

	g_Player.iMoney = (int)(g_Player.fMoney);

	if(g_Player.vy > 0.0f)
	{
		if(GetTileTypeExF(g_Player.x, g_Player.y) > 0 ||
		   GetTileTypeExF(g_Player.x - 12.0f, g_Player.y) > 0 ||
		   GetTileTypeExF(g_Player.x + 12.0f, g_Player.y) > 0)
		{
			g_Player.vy = 0.0f;
		}
	}
	else if(g_Player.vy < 0.0f)
	{
		if(GetTileTypeF(g_Player.x, g_Player.y - 64.0f) > 0 ||
		   GetTileTypeF(g_Player.x - 12.0f, g_Player.y - 64.0f) > 0 ||
		   GetTileTypeF(g_Player.x + 12.0f, g_Player.y - 64.0f) > 0)
		{
			g_Player.vy = 0.0f;
			g_Player.bJumping = FALSE;
		}
	}

	g_Player.bStanding = FALSE;
	int i1 = 0, i2 = 0, i3 = 0;
	if(g_Player.vy >= 0.0f &&
	   ((i1 = GetTileTypeExF(g_Player.x, g_Player.y + 1.0f)) > 0 ||
	    (i2 = GetTileTypeExF(g_Player.x - 12.0f, g_Player.y + 1.0f)) > 0 ||
	    (i3 = GetTileTypeExF(g_Player.x + 12.0f, g_Player.y + 1.0f)) > 0))
	{
		g_Player.bStanding = TRUE;
		g_Player.bJumping = FALSE;
	}

	i1 = GetTileTypeF(g_Player.x, g_Player.y + 1.0f);
	if(i1 == 2)
	{
		g_Player.fMoney -= 50.0f * fTime;
		fOuchCounter += fTime;
		if(fOuchCounter > 0.0f)
		{
			PlaySound(15, 16);
			fOuchCounter = -0.3f;
		}
	}
	else
	{
		fOuchCounter = 0.0f;
		if(i1 == 3) fFrictionX = 0.4f;
	}

	i1 = GetTileTypeF(g_Player.x, g_Player.y + 10.0f);
	i2 = GetTileTypeF(g_Player.x - 12.0f, g_Player.y + 10.0f);
	i3 = GetTileTypeF(g_Player.x + 12.0f, g_Player.y + 10.0f);

	if(i1 == 4 || i1 == 5 ||
	   i2 == 4 || i2 == 5 ||
	   i3 == 4 || i3 == 5)
	{
		g_Player.vx -= 100.0f * fTime;
		g_Player.y -= g_Player.vx * 0.5f * fTime;
	}
	
	if(i1 == 6 || i1 == 7 ||
	   i2 == 6 || i2 == 7 ||
	   i3 == 6 || i3 == 7)
	{
		g_Player.vx += 100.0f * fTime;
		g_Player.y += g_Player.vx * 0.5f * fTime;
	}

	if(i1 == 8)
	{
		g_Player.vx -= 100.0f * fTime;
		g_Player.y -= g_Player.vx * fTime;
	}
	else if(i1 == 9)
	{
		g_Player.vx += 100.0f * fTime;
		g_Player.y += g_Player.vx * fTime;
	}

	g_Player.x += g_Player.vx * fTime;
	g_Player.y += g_Player.vy * fTime;

	if(g_Player.vx > 0.0f)
	{
		int iNum = 0;
		fOldVX = g_Player.vx;
		while(GetTileTypeF(g_Player.x + 14.0f, g_Player.y - 8.0f) > 0 ||
			  GetTileTypeF(g_Player.x + 14.0f, g_Player.y - 32.0f) > 0 ||
			  GetTileTypeF(g_Player.x + 14.0f, g_Player.y - 58.0f) > 0)
		{
			g_Player.vx = 0.0f;
			g_Player.x -= 1.0f;
			iNum++;

			if(iNum > 0 && !(iNum % 10))
			{
				g_Player.x += 9.0f;
				g_Player.y -= 1.0f;
				g_Player.vx = fOldVX;
			}
		}
	}
	else if(g_Player.vx < 0.0f)
	{
		int iNum = 0;
		fOldVX = g_Player.vx;
		while(GetTileTypeF(g_Player.x - 14.0f, g_Player.y - 8.0f) > 0 ||
			  GetTileTypeF(g_Player.x - 14.0f, g_Player.y - 32.0f) > 0 ||
			  GetTileTypeF(g_Player.x - 14.0f, g_Player.y - 58.0f) > 0)
		{
			g_Player.vx = 0.0f;
			g_Player.x += 1.0f;
			iNum++;

			if(iNum > 0 && !(iNum % 10))
			{
				g_Player.x -= 9.0f;
				g_Player.y -= 1.0f;
				g_Player.vx = fOldVX;
			}
		}
	}

	int iNum = 0;
	while(GetTileTypeF(g_Player.x, g_Player.y - 1.0f) > 0 ||
		  GetTileTypeF(g_Player.x - 12.0f, g_Player.y - 1.0f) > 0 ||
		  GetTileTypeF(g_Player.x + 12.0f, g_Player.y - 1.0f) > 0)
	{
		g_Player.y -= 1.0f;
		iNum++;
		if(iNum > 20) {g_Player.y += 20.0f; break;}
	}

	if(GetAsyncKeyState(VK_LEFT)) {g_Player.vx -= g_fPlayerSpeed * fTime * 3.0f; g_Player.iDir = -1;}
	if(GetAsyncKeyState(VK_RIGHT)) {g_Player.vx += g_fPlayerSpeed * fTime * 3.0f; g_Player.iDir = 1;}
	
	if(GetAsyncKeyState(VK_SPACE) ||
	   GetAsyncKeyState(VK_UP) ||
	   GetAsyncKeyState(VK_LCONTROL) ||
	   GetAsyncKeyState(VK_LSHIFT))
	{
		if(g_Player.bStanding && !g_Player.bJumping && g_Player.bReadyToJump)
		{
			g_Player.vy = -250.0f * g_fPlayerJumpFactor;
			g_Player.bStanding = FALSE;
			g_Player.bJumping = TRUE;
			g_Player.bReadyToJump = FALSE;
			g_Player.fJumpCD = 0.3f;
			if(GetTileTypeF(g_Player.x, g_Player.y + 1.0f) > 0) g_Player.y -= 2.0f;
		}
		else if(!g_Player.bStanding && g_Player.bJumping && g_Player.fJumpCD > 0.0f)
		{
			g_Player.vy -= 1500.0f * g_fPlayerJumpFactor * fTime;
		}
	}
	else
	{
		g_Player.bJumping = FALSE;
		g_Player.bReadyToJump = TRUE;
	}

	if(g_Options.bUseSound)
	{
		if(g_Player.vy > 725.0f &&
		   !bAahSoundPlaying)
		{
			PlaySound(23, 23);
			bAahSoundPlaying = TRUE;
		}
		else if(g_Player.vy <= 725.0f &&
		        bAahSoundPlaying)
		{
			g_apSound[22]->Stop();
			bAahSoundPlaying = FALSE;
		}
	}

	if(g_Player.fJumpCD > 0.0f) g_Player.fJumpCD -= fTime;
	if(g_Player.vy > 0.0f) g_Player.bJumping = FALSE;

	if(fabsf(g_Player.vx) > 50.0f) g_Player.fAnim += 15.0f * fTime;
	else if(fabsf(g_Player.vx) > 30.0f) g_Player.fAnim += 10.0f * fTime;
	else if(fabsf(g_Player.vx) > 10.0f) g_Player.fAnim += 5.0f * fTime;
	else g_Player.fAnim = 0.0f;
	g_Player.fFallAnim += 8.0f * fTime;

	g_Player.vy += g_fGravity * fTime;

	fFrictionX *= 0.02f;
	g_Player.vx *= powf(fFrictionX, fTime);
	g_Player.vy *= powf(g_fFrictionY, fTime);
}

void DrawPlayer()
{
	int sx;
	int sy;


	if(g_Player.bStanding)
	{
		sx = (int)(g_Player.fAnim) % 12;
		sy = 0;
	}
	else if(g_Player.vy > 100.0f)
	{
		sx = 4;
		if(g_Player.vy > 250.0f) sx = 5;
		if(g_Player.vy > 550.0f)
		{
			if((int)(g_Player.fFallAnim) % 2) sx = 6;
			else sx = 7;
		}
		sy = 2;
	}
	else if(g_Player.vy <= 100.0f)
	{
		sx = 2;
		if(g_Player.vy < -150.0f) sx = 1;
		if(g_Player.vy < -200.0f) sx = 0;
		sy = 2;
	}

	if(g_Player.iDir == -1) sy++;

	float x = g_Player.x - g_fCameraX;
	if(g_iSnow == 2) x -= sinf(2.0f * g_fCD + (float)((g_Player.y - 32.0f) / 32.0f) * 0.25f) * 5.0f;

	DrawSprite(g_pBackBuffer, (int)(x) - 20, (int)(g_Player.y - g_fCameraY) - 80 + 1, -1, -1,
		       g_pRicardo, sx * 41, sy * 81, 40, 80, true);
}

void MoveSnow(float fTime)
{
	SSnowflake* pFlake;

	for(int i = 0; i < g_iSnowDensity; i++)
	{
		pFlake = &g_aSnowflake[i];
		if(pFlake->iType != -1)
		{
			switch(g_iSnow)
			{
			case 1:
				pFlake->x += sinf(g_fTime + (float)(i)) * 50.0f * fTime;
				pFlake->y += (100.0f + cosf(g_fTime + (float)(i)) * 25.0f) * fTime;
				break;

			case 2:
				pFlake->x += sinf(g_fTime + (float)(i)) * 30.0f * fTime;
				pFlake->y -= (80.0f + cosf(g_fTime + (float)(i)) * 25.0f) * fTime;
				break;
			}
		}
		else
		{
			pFlake->iType = rand() % g_iNumSnowSprites;

			switch(g_iSnow)
			{
			case 1:
				pFlake->x = g_fCameraX - 400.0f + (float)(rand() % 1040);
				pFlake->y = g_fCameraY - 32.0f - (float)(rand() % 32);
				break;

			case 2:
				pFlake->x = g_fCameraX - 400.0f + (float)(rand() % 1040);
				pFlake->y = g_fCameraY + 480.0f + 32.0f - (float)(rand() % 32);
				break;
			}

			pFlake->z = ((float)(10 + (rand() % 91)) / 100.0f) * 3.0f;
			pFlake->x *= pFlake->z;
			pFlake->y *= pFlake->z;
		}
	}
}

void DrawSnow(float fMinZ,
			  float fMaxZ)
{
	SSnowflake*	pFlake;
	int			px;
	int			py;


	for(int i = 0; i < g_iSnowDensity; i++)
	{
		pFlake = &g_aSnowflake[i];
		if(pFlake->iType != -1 &&
		   pFlake->z >= fMinZ &&
		   pFlake->z <= fMaxZ)
		{
			px = (int)((pFlake->x / pFlake->z) - g_fCameraX);
			py = (int)((pFlake->y / pFlake->z) - g_fCameraY);
			if(px >= -g_iSnowSpriteSize && px <= 640 + g_iSnowSpriteSize &&
			   py >= -g_iSnowSpriteSize && py <= 480 + g_iSnowSpriteSize)
			{
				DrawSprite(g_pBackBuffer, px, py, -1, -1,
					       g_pMisc, g_iSnowSpriteX + pFlake->iType * (g_iSnowSpriteSize + 1), g_iSnowSpriteY,
						   g_iSnowSpriteSize, g_iSnowSpriteSize, true);
			}
			else if(px < -200 || px > 840 ||
			        py < -100 || py > 580)
			{
				pFlake->iType = -1;
			}
		}
	}
}

void MoveItems(float fTime)
{
	SItem*	pItem;
	int		iNumItems = 0;
	int		iTile;
	int		iTileType;
	float	fOldSpeed;
	float	fSpeed;


	for(int i = 0; i < g_iTotalItems; i++)
	{
		pItem = &g_aItem[i];
		if(pItem->bExists)
		{
			if(pItem->bCaught)
			{
				pItem->vx = sinf(g_fTime * 5.0f + (float)(i)) * 150.0f;
				pItem->vy = -150.0f;
			}

			pItem->x += pItem->vx * fTime;
			pItem->y += pItem->vy * fTime;

			if(!pItem->bExploding &&
			   !pItem->bCaught)
			{
				switch(pItem->pType->iFlyingMode)
				{
				case 1:
					{
					float fSpeed;
					float fAngle;

					fSpeed = sqrtf(pItem->vx * pItem->vx + pItem->vy * pItem->vy);
					fAngle = atan2f(pItem->vy, pItem->vx);
					pItem->vx = cosf(fAngle + (fmodf((float)(i) * 0.1f + g_fTime, 5.0f) - 2.5f) * fTime) * fSpeed;
					pItem->vy = sinf(fAngle + (fmodf((float)(i) * 0.1f + g_fTime, 5.0f) - 2.5f) * fTime) * fSpeed;
					}
					break;

				case 2:
					{
					float fDist;
					float fx;
					float fy;
					float fForce;

					fx = g_Player.x - pItem->x;
					fy = (g_Player.y + 32.0f) - pItem->y;
					fForce = sqrtf(fx * fx + fy * fy);
					fx /= fForce;
					fy /= fForce;

					fDist = sqrtf((pItem->x - g_Player.x) * (pItem->x - g_Player.x) +
								  (pItem->y - (g_Player.y + 32.0f)) * (pItem->y - (g_Player.y + 32.0f)));
					if(fDist < 4.0f) fDist = 4.0f;
					fx /= fDist * 0.25f;
					fy /= fDist * 0.25f;

					pItem->vx += fx * 15000.0f * fTime;
					pItem->vy += fy * 15000.0f * fTime;
					}

				case 3:
					{
					float fDist;
					float fx;
					float fy;
					float fForce;

					fx = g_Player.x - pItem->x;
					fy = (g_Player.y + 32.0f) - pItem->y;
					fForce = sqrtf(fx * fx + fy * fy);
					fx /= fForce;
					fy /= fForce;

					fDist = sqrtf((pItem->x - g_Player.x) * (pItem->x - g_Player.x) +
								  (pItem->y - (g_Player.y + 32.0f)) * (pItem->y - (g_Player.y + 32.0f)));
					if(fDist < 2.0f) fDist = 2.0f;
					fx /= fDist * 0.5f;
					fy /= fDist * 0.5f;

					pItem->vx += fx * -2500.0f * fTime;
					pItem->vy += fy * -2500.0f * fTime;
					}
					break;

				case 5:
					{
					float fDist;
					float fx;
					float fy;
					float fForce;

					fDist = sqrtf((pItem->x - g_Player.x) * (pItem->x - g_Player.x) +
								  (pItem->y - (g_Player.y + 32.0f)) * (pItem->y - (g_Player.y + 32.0f)));
					if(fDist < 4.0f) fDist = 4.0f;
					if(fDist < 300.0f)
					{
						fx = g_Player.x - pItem->x;
						fy = (g_Player.y + 32.0f) - pItem->y;
						fForce = sqrtf(fx * fx + fy * fy);
						fx /= fForce * (1.0f + 0.2f * sinf(g_fCD));
						fy /= fForce * (1.0f + 0.2f * sinf(g_fCD));

						pItem->vx += fx * 500.0f * fTime;
						pItem->vy += fy * 500.0f * fTime;
						if(pItem->vx > 300.0f) pItem->vx = 300.0f;
						else if(pItem->vx < -300.0f) pItem->vx = -300.0f;
						if(pItem->y > g_Player.y && pItem->vy > 0.0f) pItem->vy -= 100.0f * fTime;
						else if(pItem->y < g_Player.y && pItem->vy < 0.0f) pItem->vy += 100.0f * fTime;
					}
					}
				}
			}
			
			if(!pItem->bExploding && !pItem->bCaught) pItem->fAnim += pItem->pType->fAnimSpeed * fTime;
			else
			{
				if(pItem->bExploding)
				{
					pItem->fAnim += 15.0f * fTime;
					if(pItem->fAnim >= 10.0f) pItem->bExists = FALSE;
				}
				else if(pItem->bCaught)
				{
					pItem->fAnim += fTime;
					if(pItem->fAnim >= 2.0f) pItem->bExists = FALSE;
				}
			}

			if(!pItem->bExploding &&
			   !pItem->bCaught)
			{
				iTile = GetTileF(pItem->x, pItem->y);
				iTileType = GetTileTypeF(pItem->x, pItem->y);
				if(iTileType != 0 && iTile > 0)
				{
					pItem->bExploding = TRUE;
					pItem->fAnim = 0.0f;
					pItem->vx = 0.0f;
					pItem->vy = 0.0f;
				}

				if(pItem->x > g_Player.x - 18.0f &&
				   pItem->x < g_Player.x + 18.0f &&
				   pItem->y > g_Player.y - 74.0f &&
				   pItem->y < g_Player.y)
				{
					g_Player.fMoney += (int)(pItem->iValue);

					if(pItem->pType->iFlyingMode != 4)
					{
						if(pItem->pType->iMagnet == 0)
						{
							if(pItem->iValue > 300) PlaySound(18, 18);
							else if(pItem->iValue > 0) PlaySound(1, 7);
							else if(pItem->iValue < 0) PlaySound(8, 16);
						}
						else
						{
							PlaySound(17, 17);
						}
					}
					else PlaySound(21, 21);

					pItem->bCaught = TRUE;
					pItem->fAnim = 0.0f;
					pItem->x = pItem->x + 16.0f - 30.0f;

					if(pItem->pType->iFlyingMode == 4)
					{
						g_Player.vx += pItem->vx * 4.0f;
						g_Player.vy += pItem->vy * 4.0f;
					}

					if(pItem->pType->iMagnet)
					{
						for(int j = 0; j < g_iTotalItems; j++)
						{
							if(g_aItem[j].bExists &&
							   !g_aItem[j].bExploding &&
							   !g_aItem[j].bCaught &&
							   j != i)
							{
								float fDist;
								fDist = sqrtf((pItem->x - g_aItem[j].x) * (pItem->x - g_aItem[j].x) +
									          (pItem->y - g_aItem[j].y) * (pItem->y - g_aItem[j].y));
								if(fDist > 400.0f) continue;

								if(g_aItem[j].pType->iMagnet == pItem->pType->iMagnet)
								{
									fOldSpeed = sqrtf(g_aItem[j].vx * g_aItem[j].vx + g_aItem[j].vy * g_aItem[j].vy);
									g_aItem[j].vx = ((pItem->x - 32.0f) - g_aItem[j].x) / -2.0f;
									g_aItem[j].vy = ((pItem->y - 32.0f) - g_aItem[j].y) / -2.0f;
									fSpeed = sqrtf(g_aItem[j].vx * g_aItem[j].vx + g_aItem[j].vy * g_aItem[j].vy);
									g_aItem[j].vx /= fSpeed; g_aItem[j].vx *= fOldSpeed;
									g_aItem[j].vy /= fSpeed; g_aItem[j].vy *= fOldSpeed;
								}
								else if(g_aItem[j].pType->iMagnet == -pItem->pType->iMagnet)
								{
									g_aItem[j].vx = ((pItem->x - 32.0f) - g_aItem[j].x) / 2.0f;
									g_aItem[j].vy = ((pItem->y - 32.0f) - g_aItem[j].y) / 2.0f;
								}
								else
								{
									g_aItem[j].vx = ((pItem->x - 32.0f + (float)((rand() % 65) - 32)) - g_aItem[j].x) / 2.0f * (float)(pItem->pType->iMagnet);
									g_aItem[j].vy = ((pItem->y - 32.0f + (float)((rand() % 65) - 32)) - g_aItem[j].y) / 2.0f * (float)(pItem->pType->iMagnet);
								}
							}
						}
					}
				}
			}
			
			if(pItem->x < -32.0f ||
			   pItem->y < -32.0f ||
			   pItem->x > (float)(g_iLevelWidth) * 32.0f + 32.0f ||
			   pItem->y > (float)(g_iLevelHeight) * 32.0f + 32.0f)
			{
				pItem->bExists = FALSE;
			}

			iNumItems++;
		}
	}

	if(iNumItems < g_iTotalItems)
	{
		pItem = NULL;
		for(int j = 0; j < g_iTotalItems; j++)
		{
			pItem = &g_aItem[j];
			if(!pItem->bExists) break;
		}

		if(pItem != NULL)
		{
			pItem->bExists = TRUE;
			pItem->iType = -1;
			while(pItem->iType == -1) pItem->iType = g_aiFreqTable[rand() % 100];
			pItem->pType = &g_aItemType[pItem->iType];
			pItem->iValue = pItem->pType->iValue;
			pItem->bExploding = FALSE;
			pItem->bCaught = FALSE;
			pItem->fAnim = 0.0f;

			if(pItem->pType->bRoulette)
			{
				pItem->iValue = pItem->pType->iEx1 +
					            (rand() % (pItem->pType->iEx2 - pItem->pType->iEx1 + 1));
			}

			pItem->x = g_fCameraX - 100.0f + (float)(rand() % 840);
			pItem->y = g_fCameraY - 100.0f + (float)(rand() % 680);
			if(pItem->x == g_fCameraX + 320.0f) pItem->x = g_fCameraX;
			if(pItem->y == g_fCameraY + 240.0f) pItem->y = g_fCameraY;
			while(pItem->x > g_fCameraX - 32.0f &&
				  pItem->x < g_fCameraX + 640.0f + 32.0f &&
				  pItem->y > g_fCameraY - 32.0f &&
				  pItem->y < g_fCameraY + 480.0f + 32.0f)
			{
				pItem->x += (pItem->x - (g_fCameraX + 320.0f)) * 50.0f;
				pItem->y += (pItem->y - (g_fCameraY + 240.0f)) * 50.0f;
			}

			pItem->vx = ((float)(rand() % 1001) - 500.0f) / 4.0f;
			pItem->vy = ((float)(rand() % 1001) - 500.0f) / 4.0f;
			if(fabsf(pItem->vx) < 1.0f && fabsf(pItem->vy) < 1.0f) pItem->bExists = FALSE;

			float fSpeed = sqrtf(pItem->vx * pItem->vx + pItem->vy * pItem->vy);
			if(fSpeed < g_fMinItemSpeed && fSpeed != 0.0f)
			{
				pItem->vx /= fSpeed;
				pItem->vy /= fSpeed;
				fSpeed = g_fMinItemSpeed;
				pItem->vx *= fSpeed;
				pItem->vy *= fSpeed;
			}
		}
	}
}

void DrawItems()
{
	SItem*	pItem;
	int		sx;
	int		sy;
	int		x;
	int		y;
	char	acText[256];
	HDC		hDC;


	for(int i = 0; i < g_iTotalItems; i++)
	{
		pItem = &g_aItem[i];
		if(pItem->bExists &&
		   pItem->x >= g_fCameraX - 32.0f &&
		   pItem->x <= g_fCameraX + 640.0f + 32.0f &&
		   pItem->y >= g_fCameraY - 32.0f &&
		   pItem->y <= g_fCameraY + 480.0f + 32.0f)
		{
			if(!pItem->bCaught)
			{
				if(pItem->bExploding)
				{
					sx = (int)(pItem->fAnim) % 15;
					sy = 0;
				}
				else
				{
					sx = (int)(pItem->fAnim) % pItem->pType->iNumAnimFrames;
					sy = pItem->pType->sy;
				}

				if(pItem->pType->iFlyingMode == 5) if(pItem->x > g_Player.x) sx += 2;

				float x = pItem->x;
				if(g_iSnow == 2) x -= sinf(2.0f * g_fCD + (float)(pItem->x / 32.0f) * 0.25f) * 5.0f;
				DrawSprite(g_pBackBuffer,
						   (int)(x - g_fCameraX) - 16,
						   (int)(pItem->y - g_fCameraY) - 16,
						   -1, -1,
						   g_pItems,
						   sx * 33,
						   sy * 33,
						   32,
						   32,
						   true);
			}
		}
	}

	g_pBackBuffer->GetDC(&hDC);
	SetBkMode(hDC, TRANSPARENT);
	SelectObject(hDC, g_hFont1);
	for(i = 0; i < g_iTotalItems; i++)
	{
		pItem = &g_aItem[i];
		if(pItem->bExists &&
		   pItem->x >= g_fCameraX - 32.0f &&
		   pItem->x <= g_fCameraX + 640.0f + 32.0f &&
		   pItem->y >= g_fCameraY - 32.0f &&
		   pItem->y <= g_fCameraY + 480.0f + 32.0f)
		{
			if(pItem->bCaught)
			{
				x = (int)(pItem->x - g_fCameraX);
				y = (int)(pItem->y - g_fCameraY);

				sprintf(acText, "$%d", pItem->iValue);
				if(pItem->iValue > 0)
				{
					SetTextColor(hDC, RGB(0, 127, 0)); TextOut(hDC, x - 1, y - 1, acText, strlen(acText));
					SetTextColor(hDC, RGB(0, 255, 0)); TextOut(hDC, x, y, acText, strlen(acText));
				}
				else if(pItem->iValue < 0)
				{
					SetTextColor(hDC, RGB(0, 0, 0)); TextOut(hDC, x - 1, y - 1, acText, strlen(acText));
					SetTextColor(hDC, RGB(255, 0, 0)); TextOut(hDC, x, y, acText, strlen(acText));
				}
			}
		}
	}

	g_pBackBuffer->ReleaseDC(hDC);
}

void DrawBar()
{
	HDC		hDC;
	char	acText[256];
	int		iMins;
	int		iSecs;


	g_pBackBuffer->GetDC(&hDC);
	SetBkMode(hDC, TRANSPARENT);
	SelectObject(hDC, g_hFont2);

	sprintf(acText, "$%d", g_Player.iMoney);
	if(g_Player.iMoney >= 0)
	{
		SetTextColor(hDC, RGB(0, 0, 0)); TextOut(hDC, 19, 439, acText, strlen(acText));
		SetTextColor(hDC, RGB(0, 255, 0)); TextOut(hDC, 20, 440, acText, strlen(acText));
	}
	else
	{
		SetTextColor(hDC, RGB(0, 0, 0)); TextOut(hDC, 19, 439, acText, strlen(acText));
		SetTextColor(hDC, RGB(255, 0, 0)); TextOut(hDC, 20, 440, acText, strlen(acText));
	}

	iMins = (int)(g_fCD) / 60;
	iSecs = (int)(g_fCD) % 60;
	sprintf(acText, "%0.2d:%0.2d", iMins, iSecs);
	if(iMins == 0 && iSecs <= 5)
	{
		SetTextColor(hDC, RGB(0, 0, 0));
		TextOut(hDC, 119, 439, acText, strlen(acText));
		
		if((int)(g_fCD * 2.0f) % 2) SetTextColor(hDC, RGB(0, 255, 127));
		else SetTextColor(hDC, RGB(255, 0, 0));
		TextOut(hDC, 120, 440, acText, strlen(acText));
	}
	else
	{
		SetTextColor(hDC, RGB(0, 0, 0)); TextOut(hDC, 119, 439, acText, strlen(acText));
		SetTextColor(hDC, RGB(0, 255, 127)); TextOut(hDC, 120, 440, acText, strlen(acText));
	}

	g_pBackBuffer->ReleaseDC(hDC);
}

bool HighScores(BOOL bPlayedLevel)
{
	int iChoice;

	iChoice = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_HIGHSCORES), g_hWindow, HighScoresDlgProc, bPlayedLevel);
	switch(iChoice)
	{
	case 0:
	case 2:
		MainMenu();
		break;

	case 1:
		Play();
		break;
	}

	return true;
}

bool OnlineHighScores()
{
	int iChoice;

	iChoice = DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_ONLINEHIGHSCORES), g_hWindow, OnlineHighScoresDlgProc);
	switch(iChoice)
	{
	case 0:
	case 2:
		MainMenu();
		break;

	case 1:
		Play();
		break;
	}

	return true;
}

void MoveElevators(float fTime)
{
	SElevator*	pElevator;
	float		fLength;


	for(int i = 0; i < g_iNumElevators; i++)
	{
		pElevator = &g_aElevator[i];
		if(pElevator->fTimeLeft <= 0.0f)
		{
			pElevator->x = pElevator->afPointX[pElevator->iCurrentDest];
			pElevator->y = pElevator->afPointY[pElevator->iCurrentDest];
			pElevator->iCurrentDest++;
			pElevator->iCurrentDest %= pElevator->iNumPoints;
			pElevator->vx = pElevator->afPointX[pElevator->iCurrentDest] - pElevator->x;
			pElevator->vy = pElevator->afPointY[pElevator->iCurrentDest] - pElevator->y;
			fLength = sqrtf(pElevator->vx * pElevator->vx + pElevator->vy * pElevator->vy);
			pElevator->vx /= fLength;
			pElevator->vy /= fLength;
			pElevator->vx *= pElevator->fSpeed;
			pElevator->vy *= pElevator->fSpeed;
			pElevator->fTimeLeft = fLength / pElevator->fSpeed;
		}

		if(g_Player.bStanding)
		{
			if(g_Player.x >= pElevator->x - (float)(pElevator->iWidth / 2) &&
			   g_Player.x <= pElevator->x + (float)(pElevator->iWidth / 2) &&
			   g_Player.y >= pElevator->y - (float)(pElevator->iHeight / 2) &&
			   g_Player.y <= pElevator->y + (float)(pElevator->iHeight / 2))
			{
				g_Player.y = pElevator->y - (float)(pElevator->iHeight / 2) + 2.0f;
				g_Player.x += pElevator->vx * fTime;
				g_Player.y += pElevator->vy * fTime;
			}
		}

		pElevator->x += pElevator->vx * fTime;
		pElevator->y += pElevator->vy * fTime;
		pElevator->fTimeLeft -= fTime;
	}
}

void DrawElevators()
{
	SElevator*	pElevator;
	int			px;
	int			py;


	for(int i = 0; i < g_iNumElevators; i++)
	{
		pElevator = &g_aElevator[i];
		px = (int)(pElevator->x - g_fCameraX);
		py = (int)(pElevator->y - g_fCameraY);
		
		if(px >= -pElevator->iWidth && px <= 640 + pElevator->iWidth &&
		   py >= -pElevator->iHeight && py <= 480 + pElevator->iHeight)
		{
			DrawSprite(g_pBackBuffer,
				       px - pElevator->iWidth / 2,
					   py - pElevator->iHeight / 2,
					   -1, -1,
					   g_pMisc,
					   pElevator->iSpriteX,
					   pElevator->iSpriteY,
					   pElevator->iWidth,
					   pElevator->iHeight,
					   true);
		}
	}
}

bool Play()
{
	float	fTime;
	DWORD	dwStart;
	DWORD	dwEnd;
	BOOL	bShowCursor;
	POINT	Cursor;
	POINT	OldCursor;
	char	acText[256];


	if(!g_Options.bNoHiPrio) SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);

	for(int i = 0; i < 100; i++) g_aiFreqTable[i] = -1;
	for(i = 0; i < 4096; i++) g_aSnowflake[i].iType = -1;
	ZeroMemory(g_aItem, 1024 * sizeof(SItem));

	LoadLevel(g_acLevel);
	g_Player.fAnim = 0.0f;
	g_Player.fFallAnim = 0.0f;
	g_Player.fJumpCD = 0.0f;
	g_Player.vx = 0.0f;
	g_Player.vy = 0.0f;
	g_Player.iDir = 1;
	g_Player.bStanding = FALSE;
	g_Player.bJumping = FALSE;
	g_Player.bReadyToJump = TRUE;
	g_Player.fMoney = 0.0f;
	g_Player.iMoney = 0;

	if(g_Options.bUseMusic)
	{
		sprintf(acText, "open %s alias Music", g_acMusic);
		mciSendString(acText, NULL, 0, NULL);
		mciSendString("play Music", NULL, 0, NULL);
	}

	fTime = 0.01f;
	g_fTime = 0.0f;
	g_dwLastMouseMove = timeGetTime();
	bShowCursor = TRUE;
	GetCursorPos(&Cursor);
	OldCursor = Cursor;
	static BOOL bTimeOverSignalPlayed;

	bTimeOverSignalPlayed = FALSE;

	while(true)
	{
		OldCursor = Cursor;
		GetCursorPos(&Cursor);
		if(abs(Cursor.x - OldCursor.x) >= 3 ||
		   abs(Cursor.y - OldCursor.y) >= 3) g_dwLastMouseMove = timeGetTime();

		dwStart = timeGetTime();
		
		if(timeGetTime() > g_dwLastMouseMove + 1000)
		{
			if(bShowCursor)
			{
				ShowCursor(FALSE);
				bShowCursor = FALSE;
			}
		}
		else
		{
			if(!bShowCursor)
			{
				ShowCursor(TRUE);
				bShowCursor = TRUE;
			}
		}

		if(!ProcessMessages()) break;

		MovePlayer(fTime);
		MoveElevators(fTime);
		MoveItems(fTime);
		if(g_Options.bShowSnow && g_iSnow) MoveSnow(fTime);

		g_fCameraX = g_Player.x - 320.0f;
		g_fCameraY = g_Player.y - 240.0f;
		if(g_fCameraX < 0.0f) g_fCameraX = 0.0f;
		if(g_fCameraY < 0.0f) g_fCameraY = 0.0f;
		if(g_fCameraX + 640.0f >= (float)(g_iLevelWidth * 32)) g_fCameraX = (float)(g_iLevelWidth * 32) - 640.0f;
		if(g_fCameraY + 480.0f >= (float)(g_iLevelHeight * 32)) g_fCameraY = (float)(g_iLevelHeight * 32) - 480.0f;

		if(!g_Options.bShowSky) ClearSurface(g_pBackBuffer, 0);
		else DrawSky();
		if(g_Options.bShowSnow && g_iSnow) DrawSnow(1.0f, 100.0f);
		DrawLevel();
		DrawElevators();
		DrawPlayer();
		DrawItems();
		if(g_Options.bShowSnow && g_iSnow) DrawSnow(0.0f, 1.0f);
		DrawBar();
		ShowBackBuffer(true);

		dwEnd = timeGetTime();
		fTime = (float)(dwEnd - dwStart) / 1000.0f;
		g_fTime += fTime;
		g_fCD -= fTime;

		if(g_fCD <= 5.0f)
		{
			if(!bTimeOverSignalPlayed) PlaySound(22, 22);
			bTimeOverSignalPlayed = TRUE;
		}

		if(g_fCD < 0.0f) break;
		if(g_Player.y > (float)(g_iLevelHeight) * 32.0f + 300.0f)
		{
			PlaySound(19, 20);
			break;
		}

		if(GetAsyncKeyState(VK_ESCAPE))
		{
			if(!bShowCursor)
			{
				ShowCursor(TRUE);
				bShowCursor = TRUE;
			}

			if(MessageBox(g_hWindow, g_bEnglish ? "Do you want to abort the game?" : "Mchten Sie das laufende Spiel wirklich abbrechen?", g_bEnglish ? "Confirm" : "Abbruch besttigen",
				          MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) == IDYES) break;
			g_dwLastMouseMove = timeGetTime();
		}

		if(GetAsyncKeyState(VK_PAUSE))
		{
			if(!bShowCursor)
			{
				ShowCursor(TRUE);
				bShowCursor = TRUE;
			}

			MessageBox(g_hWindow, g_bEnglish ? "The game has been paused.\nClick \"OK\" to continue." : "Das Spiel wurde angehalten.\nKlicken Sie auf\"OK\", um weiterzuspielen.", "Pause", MB_OK | MB_ICONINFORMATION);
			g_dwLastMouseMove = timeGetTime();
		}
	}

	if(g_Options.bUseSound) g_apSound[21]->Stop();
	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);

	if(!bShowCursor)
	{
		ShowCursor(TRUE);
		bShowCursor = TRUE;
	}

	SAFE_RELEASE(g_pTiles);
	SAFE_RELEASE(g_pSky1);
	SAFE_RELEASE(g_pSky2);
	SAFE_RELEASE(g_pItems);

	if(g_Options.bUseMusic)
	{
		mciSendString("stop Music", NULL, 0, NULL);
		mciSendString("close Music", NULL, 0, NULL);
	}

	HighScores(TRUE);

	return true;
}

bool SelectLevel()
{
	int iChoice;

	iChoice = DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_SELECTLEVEL), g_hWindow, SelectLevelDlgProc);
	switch(iChoice)
	{
	case 0:
	case 2:
		MainMenu();
		break;

	case 1:
		Play();
		break;
	}

	return true;
}

bool MainMenu()
{
	int		iChoice;
	char	acSound[256];


Start:
	DrawSprite(g_pBackBuffer, 0, 0, -1, -1, g_pTitle, 0, 0, 640, 480, false);
	ShowBackBuffer(true);

	iChoice = DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_MAINMENU), g_hWindow, MainMenuDlgProc);
	switch(iChoice)
	{
	case 1:
		SelectLevel();
		break;

	case 2:
		DialogBox(GetModuleHandle(NULL),
				  MAKEINTRESOURCE(IDD_HELP),
				  g_hWindow,
				  HelpDlgProc);
		goto Start;
		break;

	case 3:
		HighScores(FALSE);
		break;

	case 4:
		OnlineHighScores();
		break;

	case 5:
		DialogBox(GetModuleHandle(NULL),
				  MAKEINTRESOURCE(IDD_OPTIONS),
				  g_hWindow,
				  OptionsDlgProc);
		if(g_Options.bUseSound &&
		   g_apSound[0] == NULL)
		{
			InitDirectSound(g_hWindow);
			for(int i = 0; i < g_iNumSounds; i++)
			{
				sprintf(acSound, "Data\\Sound%d.wav", i + 1);
				LoadSound(acSound, DSBCAPS_STATIC | DSBCAPS_CTRLFREQUENCY, &g_apSound[i]);
			}
		}
		goto Start;
		break;
	}

	return true;
}

void CreateFonts()
{
	LOGFONT f;

	ZeroMemory(&f, sizeof(LOGFONT));
	f.lfHeight = 24;
	f.lfWidth = 10;
	f.lfEscapement = 0;
	f.lfOrientation = 0;
	f.lfWeight = 600;
	f.lfCharSet = ANSI_CHARSET;
	f.lfOutPrecision = OUT_DEFAULT_PRECIS;
	f.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	f.lfQuality = ANTIALIASED_QUALITY;
	f.lfPitchAndFamily = DEFAULT_PITCH;
	strcpy(f.lfFaceName, "Tahoma");
	g_hFont1 = CreateFontIndirect(&f);

	g_hFont2 = CreateFontIndirect(&f);
}

int WINAPI WinMain(HINSTANCE hInst,
				   HINSTANCE hPrevInst,
				   char* pcCmdLine,
				   int iShowCmd)
{
	FILE*	pFile;
	char	acSound[256];


	srand(timeGetTime() % 10000);

	g_hCursor = LoadCursor(hInst, MAKEINTRESOURCE(IDC_CURSOR1));
	g_hWaitCursor = LoadCursor(hInst, MAKEINTRESOURCE(IDC_CURSOR2));

	pFile = fopen("Data\\Ricardo2.cfg", "rb");
	if(pFile == NULL)
	{
		g_Options.bShowSky = TRUE;
		g_Options.bShowSnow = TRUE;
		g_Options.bUseSound = TRUE;
		g_Options.bUseMusic = TRUE;
		g_Options.bNoHiPrio = FALSE;
	}
	else
	{
		fread(&g_Options, sizeof(SOptions), 1, pFile);
		fclose(pFile);
	}

	InitWindow(640, 480, "Ricardo 2", "Ricardo 2",
		       LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON1)), g_hCursor);
	InitDirectDraw(640, 480, 16, g_hWindow);
	if(g_Options.bUseSound) InitDirectSound(g_hWindow);

	LoadBitmap("Data\\Title.bmp", &g_pTitle, 0);
	LoadBitmap("Data\\Ricardo.bmp", &g_pRicardo, 0);
	LoadBitmap("Data\\Misc.bmp", &g_pMisc, 0);
	CreateFonts();
	if(g_Options.bUseSound)
	{
		for(int i = 0; i < g_iNumSounds; i++)
		{
			sprintf(acSound, "Data\\Sound%d.wav", i + 1);
			LoadSound(acSound, DSBCAPS_STATIC | DSBCAPS_CTRLFREQUENCY, &g_apSound[i]);
		}
	}

	MainMenu();

	for(int i = 0; i < g_iNumSounds; i++) SAFE_RELEASE(g_apSound[i]);
	SAFE_RELEASE(g_pTitle);
	SAFE_RELEASE(g_pRicardo);
	SAFE_RELEASE(g_pMisc);
	if(g_hFont1 != NULL) {DeleteObject(g_hFont1); g_hFont1 = NULL;}
	if(g_hFont2 != NULL) {DeleteObject(g_hFont2); g_hFont2 = NULL;}
	ExitDirectSound();
	ExitDirectDraw();
	ExitWindow();

	return 0;
}