// Include-Dateien
#include "Window.h"
#include "Globals.h"
#include "DirectDraw.h"

// Globale Variablen
HWND		g_hWindow;
WNDCLASSEX	g_WC;

// Erstellt das Fenster
bool InitWindow(int iWidth,
				int iHeight,
				char* pcClass,
				char* pcTitle,
				HICON hIcon,
				HCURSOR hCursor)
{
	// Fensterklasse ausfllen
	WNDCLASSEX WindowClass = {sizeof(WNDCLASSEX), CS_CLASSDC, WindowProc, 0, 0,
							  GetModuleHandle(NULL), hIcon, hCursor, NULL, NULL, pcClass, NULL};

	// Fensterklasse registrieren
	g_WC = WindowClass;
	if(!RegisterClassEx(&WindowClass)) return false;

	// Fenster erstellen
	g_hWindow = CreateWindow(pcClass,
							 pcTitle,
							 WS_POPUP | WS_VISIBLE,
							 GetSystemMetrics(SM_CXSCREEN) / 2 - iWidth / 2,
							 GetSystemMetrics(SM_CYSCREEN) / 2 - iHeight / 2,
							 iWidth,
							 iHeight,
							 NULL,
							 NULL,
							 GetModuleHandle(NULL),
							 NULL);
	if(g_hWindow == NULL) return false;

	return true;
}

// Fhrt das Fenster herunter
bool ExitWindow()
{
	DestroyWindow(g_hWindow);
	UnregisterClass(g_WC.lpszClassName, GetModuleHandle(NULL));

	return true;
}

// Nachrichtenschleife
bool ProcessMessages()
{
	MSG Msg;

	ZeroMemory(&Msg, sizeof(MSG));
	while(PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE))
	{
		TranslateMessage(&Msg);
		DispatchMessage(&Msg);
		if(Msg.message == WM_QUIT) return false;
	}

	return true;
}

// Nachrichtenfunktion
LRESULT WINAPI WindowProc(HWND hWindow,
						  unsigned int uiMsg,
						  WPARAM WParam,
						  LPARAM LParam)
{
	PAINTSTRUCT Paint;
	HDC hBBDC;


	switch(uiMsg)
	{
	case WM_CREATE:
		break;

	case WM_SETCURSOR:
		SetCursor(g_hCursor);
		break;

	case WM_PAINT:
		// Back-Buffer sichtbar machen
		if(g_pBackBuffer != NULL)
		{
			BeginPaint(hWindow, &Paint);
			g_pBackBuffer->GetDC(&hBBDC);
			BitBlt(Paint.hdc, 0, 0, 640, 480, hBBDC, 0, 0, SRCCOPY);
			g_pBackBuffer->ReleaseDC(hBBDC);
			EndPaint(hWindow, &Paint);
		}
		break;

	case WM_CLOSE:
		DestroyWindow(hWindow);
		PostQuitMessage(0);
		break;

	default:
		// Standardverarbeitung der Nachricht
		return DefWindowProc(hWindow, uiMsg, WParam, LParam);
	}

	return TRUE;
}